#!/bin/bash

####################################################################################################################################################################
# 集群信息收集工具
# 用法：
# 不传递参数默认将此脚本分发至集群环境的所有节点/home目录下
# 可通过传递参数只执行某一块信息的查询
# 执行完毕后会在/home目录下产生所有节点反馈回的信息压缩包
####################################################################################################################################################################

SCRIPT_NAME=$0
CURRENT_DIR="$(cd "$( dirname "${BASH_SOURCE[0]}" )" >/dev/null 2>&1 && pwd)"
COLLECTOR_TOOL_PATH=${CURRENT_DIR}/${SCRIPT_NAME}
#该SLAVE_IP实际不是从机IP，但因已有多处引用，不进行变量名修改，之后引用时请注意
SLAVE_IP=$(dbcfgtool.sh  -q | grep slave= | sed 's/,/ /g' | awk -F = '{print $2}' | awk '{print $1}')
MASTER_IP=$(dbcfgtool.sh  -q | grep master= | awk -F = '{print $2}')
BROKERS="$(cat /bigdata/salut/conf/db_bigdata.conf | grep 'slave=' | cut -d '=' -f 2 | sed 's/,/:9092,/g'):9092"
ZKCONNECT="$(cat /bigdata/salut/components/kafka/config/server.properties | grep ^"zookeeper.connect=" | cut -d = -f 2)"
PGPASSWORD=$(cat /bigdata/salut/components/jetty/resources/application.properties | grep ^"jdbc.password")
DBEVERSION=$(/bigdata/salut/bin/common/dbcfgtool.sh -v | grep 'Exterior version')
if grep -q "IMOS110" <<< "$DBEVERSION"; then
    DBEVERSION=$(echo "$DBEVERSION" | cut -d '-' -f 3 | cut -d '.' -f 1 | cut -d 'B' -f 2)
else
    DBEVERSION=$(echo "$DBEVERSION" | cut -d '-' -f 2 | cut -d '.' -f 1 | cut -d 'B' -f 2)
fi
GP_PSQL=/bigdata/salut/components/postgres/pgsql/bin/psql
export PGPASSWORD=$(unipd_64 1 "${PGPASSWORD#jdbc.password=}")
source /bigdata/salut/bin/common/bdcomm.sh
getDBEinfo
ALL_NODES=$(echo $ALL_NODES | sed 's/,/ /g')
BIGDATA=/bigdata/salut
VAR_CONF="$BIGDATA"/conf
CFGCHG_BIGDATA="$VAR_CONF"/db_bigdata.conf
CFGCHG_GLOBAL="$VAR_CONF"/db_global.conf
SLAVES=$(cat ${CFGCHG_BIGDATA} | grep slave | awk -F '=' '{print $2}' | sed 's/,/\n/g')
if grep -q "^node" $CFGCHG_BIGDATA; then
    ALL_HOST_IP=$(grep "^node" $CFGCHG_BIGDATA)
else
    ALL_HOST_IP=""
    while IFS= read -r single_ip; do
        single_node=$(ssh -n ${single_ip} "cat ${CFGCHG_GLOBAL} | grep devCode" | awk -F '=' '{print $2}')
        if [ -z "$ALL_HOST_IP" ]; then
            ALL_HOST_IP="${single_node}=${single_ip}"
        else
            ALL_HOST_IP="${ALL_HOST_IP}"$'\n'"${single_node}=${single_ip}"
        fi
    done <<< "$SLAVES"
fi
BIN_COMMON_PATH=/bigdata/salut/bin/common
GP_BAK_TOLL_PATH=$BIN_COMMON_PATH/greenplumBak.sh
GP_BAK_TOLL_RENAME=$BIN_COMMON_PATH/greenplumBakCheck$(date +%Y-%m-%d).sh
APPLICATION_PROPERTIES_PATH=/bigdata/salut/components/jetty/resources/application.properties
HOSTNAME=$(hostname)
if [ -z "$SLAVE_NUM" ]; then
    SLAVE_NUM=$(cat ${CFGCHG_BIGDATA} | grep slave | sed 's/,/\n/g'| sed '/^$/d' | wc -l)
fi


####################################################################################################################################################################
# 配置项
####################################################################################################################################################################
IOTOP_COLLECT_NUM=15
# 超过200GB的文件需要输出
# HADOOP内文件大小限制200GB
MIX_B_SIZE=214748364800
# Linux系统内文件大小限制200GB
MIX_K_SIZE=209715200
MAX_LOG_FILE_IN_KB=204800
MAX_DISK_CAPACITY=80
CHECK_COUNT=5
CHECK_DELAY_IN_SEC=30
BACKUPMAG=/bigdata/backupMsg
# 收集文件存放地址
OUTPUT_PATH=/home
MASTER_INFO=master-info
OUTPUT_PATH_MASTER=${OUTPUT_PATH}/${MASTER_INFO}
OUTPUT_PATH_CONFIG=${OUTPUT_PATH_MASTER}/01-日志与配置信息
OUTPUT_NODE_PATH=${OUTPUT_PATH_MASTER}/02-节点详细信息
OUTPUT_PATH_COMPONENTS=${OUTPUT_PATH_MASTER}/03-全部组件信息
OUTPUT_PATH_UPDATE_ACCESS=${OUTPUT_PATH}/${MASTER_INFO}/04-升级评估
OUTPUT_FILE_CLUSTER_INFO=${OUTPUT_PATH_MASTER}/05-巡检结果.txt
OUTPUT_SOFTWARE_INFO=${OUTPUT_PATH_MASTER}/06-软件信息.txt
OUTPUT_HARDWARE_INFO=${OUTPUT_PATH_MASTER}/07-硬件信息.txt
OUTPUT_STATISTICS_FILE=${OUTPUT_PATH_MASTER}/08-数据量统计.txt
OUTPUT_UPDATE_ACCESS_FILE=${OUTPUT_PATH_UPDATE_ACCESS}/01-升级评估收集信息.txt
OUTPUT_FILE_ES=${OUTPUT_PATH_COMPONENTS}/01-ES信息.txt
OUTPUT_FILE_HBASE=${OUTPUT_PATH_COMPONENTS}/02-HBASE信息.txt
OUTPUT_FILE_HDFS=${OUTPUT_PATH_COMPONENTS}/03-HDFS信息.txt
OUTPUT_FILE_TTL=${OUTPUT_PATH_COMPONENTS}/04-留存期信息.txt
OUTPUT_FILE_GP=${OUTPUT_PATH_COMPONENTS}/05-GP信息.txt
OUTPUT_FILE_REDIS=${OUTPUT_PATH_COMPONENTS}/06-REDIS信息.txt
OUTPUT_FILE_YARN=${OUTPUT_PATH_COMPONENTS}/07-YARN信息.txt
OUTPUT_FILE_KAFKA=${OUTPUT_PATH_COMPONENTS}/08-KAFKA信息.txt

SERVER_LOG_DIR="服务类日志"
SERVICE_LOG_DIR="业务类日志"
SYSTEM_LOG_DIR="系统日志"

OUTPUT_TAR=${OUTPUT_PATH}/bigdata-db-collector-$(date +%Y-%m-%d-%H-%M-%S).tgz
# 主机监测异常信息报告

# 环境信息


OUTPUT_FILE_SUCCESS_INFO_TEMP=${OUTPUT_PATH_MASTER}/success_temp.txt
OUTPUT_FILE_ERROR_INFO_TEMP=${OUTPUT_PATH_MASTER}/error_temp.txt

# 系统日志收集
PATH_OF_SA=${OUTPUT_PATH_MASTER}/sa
PATH_OF_MESSAGE=${OUTPUT_PATH_MASTER}/message
PATH_OF_CRON=${OUTPUT_PATH_MASTER}/cron
PATH_OF_DMESG=${OUTPUT_PATH_MASTER}/dmesg
OUTPUT_LOG=${OUTPUT_PATH_MASTER}/collector.log
FINISH_FLAG_FILE=${OUTPUT_PATH_MASTER}/finish_flag.txt
YARN_PROPERTIES_FILE=/bigdata/salut/components/hadoop/etc/hadoop/yarn-site.xml



# 大数据各服务的端口号
Zookeeper=(2181)
HDFS_NameNode=(50070)
YARN_PORT=$(cat ${YARN_PROPERTIES_FILE} | grep -A 1 yarn.resourcemanager.webapp.address | grep -v address | awk -F '<' '{print $2}' | awk -F '>' '{print $2}' | awk -F : '{print $2}')
Yarn_ResourceManager=($YARN_PORT)
HBase_Master=(16010)
Elasticsearch=(9200)
Kibana=(5601)
Kafka=(9092)
PostgreSQL=(5432)
GreenPlum=(6543)
MasterServer=(19098)
SlaveServer=(9099)
IrisServer=(9100)
FeatureSlave=(18000)
FeatureMaster=(18001)
Jetty=(4051 80)
Nginx=(8094)
Bgclient=(8103)

#采用关联数组方式定义一个二维数组
declare -A serverArray=([Zookeeper]="${Zookeeper[@]}" [HDFS_NameNode]="${HDFS_NameNode[@]}" [Yarn_ResourceManager]="${Yarn_ResourceManager[@]}" [HBase_Master]="${HBase_Master[@]}" [Elasticsearch]="${Elasticsearch[@]}" [Kibana]="${Kibana[@]}" [Kafka]="${Kafka[@]}" [PostgreSQL]="${PostgreSQL[@]}" [GreenPlum]="${GreenPlum[@]}" [MasterServer]="${MasterServer[@]}" [SlaveServer]="${SlaveServer[@]}" [IrisServer]="${IrisServer[@]}" [FeatureSlave]="${FeatureSlave[@]}" [FeatureMaster]="${FeatureMaster[@]}" [Jetty]="${Jetty[@]}" [Nginx]="${Nginx[@]}" [Bgclient]="${Bgclient[@]}")

FLUME_CONF_PROPERTIES=/bigdata/salut/components/flume/conf/flume-conf.properties
MIN_WINDOW_SIZE=50

LENGTH_FIRST_COLUMN=18
LENGTH_LAST_ERROR_COLUMN=10



CURRENT_WINDOW_SIZE=$(resize 2>/dev/null)
if [ $? -ne 0 ]; then
    CURRENT_WINDOW_SIZE=$(stty size 2>/dev/null)
    if [ $? -ne 0 ]; then
        CURRENT_WINDOW_SIZE=100
    else
        CURRENT_WINDOW_SIZE=$(echo "$CURRENT_WINDOW_SIZE" | awk '{print $2}')
    fi
else
    CURRENT_WINDOW_SIZE=$(echo "$CURRENT_WINDOW_SIZE" | grep COLUMNS= | cut -d = -f 2 | cut -d ';' -f 1)
fi
length_success_array=($LENGTH_FIRST_COLUMN $((CURRENT_WINDOW_SIZE - LENGTH_FIRST_COLUMN - 3)))
length_error_array=($LENGTH_FIRST_COLUMN $((CURRENT_WINDOW_SIZE - LENGTH_FIRST_COLUMN - LENGTH_LAST_ERROR_COLUMN - 4)) $LENGTH_LAST_ERROR_COLUMN)

success_table_head=(" 检测项 " " 检测结果 ")
error_table_head=(" 检测项 " " 检测结果 " " 处理方式 ")

# 检测项 检测正常描述 检测异常描述 处理方式 检测函数
NODE_ONLINE_STATUS=(" 节点在线情况 " " %s节点在线 " " %s节点离线 " " 必须处理 " "check_node_online_status")
SERVER_RUNNING_STATUS=(" 服务运行情况 " " %s节点服务均正常 " " %s节点%s服务异常 " " 必须处理 " "null")
HDFS_COMPONENTS_STATUS=(" HDFS组件状态 " " HDFS组件正常（%s） " " HDFS组件异常（%s） " " 必须处理 " "null")
HDFS_CORRUPT_BLOCK_STATUS=(" HDFS坏块情况 " " HDFS不存在坏块 " " HDFS存在%s个坏块 " " 必须处理 " "check_hdfs_corrupt_block_status")
HBASE_COMPONENTS_STATUS=(" HBASE组件状态 " " HBASE组件正常（%s） " " HBASE组件异常（%s） " " 必须处理 " "check_hbase_components_status")
HBASE_CLUSTER_LOAD_STATUS=(" HBASE集群负载 " " HBASE集群平均负载：%s，集群总负载：%s " " 无异常 " " 需要关注 " "check_hbase_cluster_load_status")
GREENPLUM_COMPONENTS_STATUS=(" GP组件状态 " " segment共%d个，在线%d个 " " segment共%d个，在线%d个 " " 必须处理 " "check_greenplum_components_status")
REDIS_COMPONENTS_STATUS=(" Redis组件状态 " " Redis组件正常 " " Redis组件异常 " " 必须处理 " "check_redis_components_status")
KAFKA_TOPIC_STATUS=(" Topic情况 " " 不存在缺失的Topic " " 存在缺失的Topic：%s " " 必须处理 " "null")
CONSUMER_GROUP_STATUS=(" 消费者组情况 " " 不存在缺失的消费者组 " " 存在缺失的消费者组：%s " " 必须处理 " "check_consumer_group_status")
FLUME_CONSUMER_STATUS=(" FLUME消费情况 " " 不存在阻塞的消费者组 " " 存在阻塞的消费者组：%s " " 必须处理 " "null")
TTL_STATUS=(" 留存期情况 " " 数据类型=(实际存储天数/配置留存期)：%s " "数据类型=(实际存储天数/配置留存期)：%s" " 必须处理 " "check_topic_and_ttl_status")
ES_DIV_CORE_STATUS=(" es分core情况 " " 不存在异常的分core索引 " "%s索引本应存在的core为%s,实际存在的core为%s" " 必须处理 " "check_es_div_core_status")
ES_LACK_INDEX_STATUS=(" ES索引状态 " " 不存在缺失的索引 " " 存在缺失的索引：%s " " 必须处理 " "null")
ES_INDEX_STATUS=(" ES索引状态 " " ES索引状态均正常（green） " "以下索引状态均为%s：%s" " 必须处理 " "check_es_index_status")
ES_CLUSTER_STATUS=(" ES集群状态 " " ES集群状态正常（green） " " ES集群状态异常（%s） " " 必须处理 " "check_es_cluster_status")
ES_INDEX_RO_STATUS=(" ES索引读写 " " ES索引读写正常 " " ES中以下索引只能读取，不能写入：%s " " 必须处理 " "null")
ES_SHARD_STATUS=(" ES分片状态 " " shard共%d个，存活%d个 " " shard共%d个，存活%d个 " " 必须处理 " "check_es_shard_status")
PATCH_STATUS=(" 补丁状态 " " 检测到环境中存在补丁：%s " " 环境中不存在补丁，请联系研发评估是否需要打通用补丁 " " 需要关注 " "check_patch_status")
PATCH_RECORD_STATUS=(" 补丁记录 " " 补丁安装顺序：%s " " 无异常 " " 需要关注 " "null")
DISK_UPPER_STATUS=(" 磁盘存储情况 " " 不存在使用率超过75%的磁盘 " " 存在使用率超过75%%的磁盘：%s " " 必须处理 " "check_disk_upper_status")
DISK_READ_WRITE_STATUS=(" 磁盘读写情况 " " 不存在读写异常的磁盘 " " 存在只有读取权限的磁盘：%s " " 必须处理 " "null")
SSH_COPY_ID_STATUS=(" ssh-copy-id情况 " " ssh-copy-id文件无缺失且权限正常 " " 无异常 " " 必须处理 " "check_ssh_copy_id_status")
BACKUP_MSG_DISK_STATUS=(" 冷备盘情况 " " 自定义 " " 自定义 " " 自定义 " "check_backup_msg_status")
DEVICE_SERIAL_NUMBER_STATUS=( " 设备SN码 " " 设备SN码如下(ip->SN码)：%s " " 仅支持获取自研服务器的设备SN码 " " 无需处理 " "check_device_serial_number_status")
UPDATE_BEFORE_VERSION_STATUS=( " 升级前提醒 " " 无 " " 目前版本小于3339版本，如果想升级到大于等于B3339版本，请务必备份数据，升级后需要清洗数据！ " " 按需处理 " "check_update_before_version_status")

declare -A check_array=([0]="${NODE_ONLINE_STATUS[@]}" [1]="${HDFS_CORRUPT_BLOCK_STATUS[@]}"
[2]="${HBASE_COMPONENTS_STATUS[@]}" [3]="${HBASE_CLUSTER_LOAD_STATUS[@]}" [4]="${GREENPLUM_COMPONENTS_STATUS[@]}"
[5]="${REDIS_COMPONENTS_STATUS[@]}" [6]="${CONSUMER_GROUP_STATUS[@]}" [7]="${TTL_STATUS[@]}"
[8]="${ES_DIV_CORE_STATUS[@]}" [9]="${ES_INDEX_STATUS[@]}" [10]="${ES_CLUSTER_STATUS[@]}" [11]="${ES_SHARD_STATUS[@]}"
[12]="${PATCH_STATUS[@]}" [13]="${DISK_UPPER_STATUS[@]}" [14]="${SSH_COPY_ID_STATUS[@]}" [15]="${BACKUP_MSG_DISK_STATUS[@]}"
[16]="${DEVICE_SERIAL_NUMBER_STATUS[@]}" [17]="${UPDATE_BEFORE_VERSION_STATUS[@]}")

DATA_METRIC_LENGTH=12

PRINT_STAT_WIDTH=$(((CURRENT_WINDOW_SIZE - 1) / 13))
if [ $PRINT_STAT_WIDTH -gt 7 ]; then
    PRINT_STAT_WIDTH=7
fi

if [ -z "$(dbserver.sh status_detail | grep "The wrong parameter")" ]; then
    is_lower_version=false
else
    is_lower_version=true
fi

print_log() {
    local msg=$1
    echo "[$(date +'%Y-%m-%d %H:%M:%S')] $msg"
}

print_separate_rows(){
    echo
    echo
    echo
}

get_statistics_date_by_week(){
    local start_date=$1
    # 获取30天前的日期
    local before_30_date=$(date -d "-30 day" +%Y-%m-%d)
    local start_seconds=$(date -d "$start_date" +%s)
    local before_30_seconds=$(date -d "$before_30_date" +%s)
    if [ $start_seconds -lt $before_30_seconds ]; then
        start_date=$before_30_date
        start_seconds=$before_30_seconds
    fi
    local print_date=""

    local now=$(date +%s)
    local i=0
    while [ $start_seconds -le $now ]; do
        start_date=$(date -d @$start_seconds +%Y-%m-%d)
        print_date="${print_date}${start_date} "
        ((i+=1))
        local index_in_week=$(date -d "$start_date" +%u)
        if (( i == PRINT_STAT_WIDTH && start_seconds < now )); then
            print_date="${print_date% }"
            print_date="${print_date}\n"
            i=0
        fi
        ((start_seconds+=86400))
    done
    echo -e "$print_date"
}

get_statistics_date_by_month(){
    local start_date=$1
    # 获取5月前的日期
    local before_5_month_date="$(date -d "-5 month" +%Y-%m)-01"
    local start_seconds=$(date -d "$start_date" +%s)
    local before_5_month_seconds=$(date -d "$before_5_month_date" +%s)
    if [ $start_seconds -lt $before_5_month_seconds ]; then
        start_date=$before_5_month_date
        start_seconds=$before_5_month_seconds
    fi
    local print_date=""
    local i
    local now=$(date +%s)
    for (( i=0; i<6; i++ )); do
        local cur_date
        if [ $i -eq 0 ]; then
            print_date="${print_date}${start_date} "
        else
            local month=$(date -d "$(date -d "${start_date}" +%Y-%m)-01 +${i} month" +%Y-%m-%d)
            local month_ms=$(date -d "$month" +%s)
            if [ $month_ms -gt $now ]; then
                break
            elif (( i%PRINT_STAT_WIDTH==0)); then
                print_date="${print_date% }"
                print_date="${print_date}\n"
            fi
            print_date="${print_date}${month} "
        fi
    done
    print_date="${print_date% }"
    echo "$print_date"

}

print_str_sequence() {
    local str=$1
    local count=$2
    local i
    for ((i=0; i<$count; i++)); do
        echo -n "$str"
    done
}

cal_size() {
    local str=$1
    #去掉颜色
    str=$(echo "$str" | sed 's/\x1b\[[0-9;]*m//g')
    #统计中文字符的数量（标点符号只统计常见的）
    local zh_size=$(echo "$str" | grep -o -E '[一-龥]|[，]|[。]|[：]|[【]|[】]|[？]|[！]|[（]|[）]' | wc -l)

    local en_size=$((${#str} - zh_size))

    local str_size=$((zh_size * 2 + en_size))

    echo "$str_size"
}

# 连接PG数据库，用于收集表信息
exec_sql() {
    sql=$1
    if [ -f "/bigdata/salut/components/postgres/pgsql/bin/psql" ]; then
        /bigdata/salut/components/postgres/pgsql/bin/psql -U postgres -d bigdata -c "$sql"
        if [ $? -ne 0 ]; then
            return 1
        fi
    else
        /home/postgres/pgsql/bin/psql -U postgres -d bigdata -c "$sql"
        if [ $? -ne 0 ]; then
            return 1
        fi
    fi
}


merge_success_items(){
    if [ -f "$GP_BAK_TOLL_PATH" ];then
        mv "$GP_BAK_TOLL_PATH" "$GP_BAK_TOLL_RENAME"
    fi
    # 打印正常信息表头
    echo -e "\033[32m检测到正常的信息：\033[0m"
    local i
    for i in "${!length_success_array[@]}"; do
        echo -n "+"
        print_str_sequence "-" ${length_success_array[i]}
    done
    echo "+"
    for i in "${!length_success_array[@]}"; do
        echo -n "|${success_table_head[$i]}"
        local white_space_length=$((length_success_array[i] - $(cal_size "${success_table_head[$i]}")))
        print_str_sequence " " $white_space_length
    done
    echo "|"
    for i in "${!length_success_array[@]}"; do
        if [ $i -eq 0 ]; then
            echo -n "|"
        else
            echo -n "+"
        fi
        print_str_sequence "=" ${length_success_array[i]}
    done
    echo -n "|"

    for ((i=0; i<${#check_array[@]}; i++));do
        local check=(${check_array[$i]})
        local func=${check[4]}
        "$func"
    done
    echo -ne "\033[1K\r"
    for i in "${!length_success_array[@]}"; do
        echo -n "+"
        print_str_sequence "-" ${length_success_array[i]}
    done
    echo "+"
    echo
}

merge_error_message(){
    if [ -f "$OUTPUT_FILE_ERROR_INFO_TEMP" ]; then
        # 打印异常信息表头
        echo -e "\033[31m检测到异常的信息：\033[0m"
        for i in "${!length_error_array[@]}"; do
            echo -n "+"
            print_str_sequence "-" ${length_error_array[i]}
        done
        echo "+"
        for i in "${!length_error_array[@]}"; do
            echo -n "|${error_table_head[$i]}"
            local white_space_length=$((length_error_array[i] - $(cal_size "${error_table_head[$i]}")))
            print_str_sequence " " $white_space_length
        done
        echo "|"
        for i in "${!length_error_array[@]}"; do
            if [ $i -eq 0 ]; then
                echo -n "|"
            else
                echo -n "+"
            fi
            print_str_sequence "=" ${length_error_array[i]}
        done
        echo "|"
        sed '1d' "$OUTPUT_FILE_ERROR_INFO_TEMP" | sed '$d'
        rm -f "$OUTPUT_FILE_ERROR_INFO_TEMP"
        # 打印表格最后一行
        for i in "${!length_error_array[@]}"; do
            echo -n "+"
            print_str_sequence "-" ${length_error_array[i]}
        done
        echo "+"
    else
        echo -e "\033[32m未检测到异常信息\033[0m"
    fi
    # 拉起daemon
    dbserver.sh start_server_all daemon > /dev/null
    echo
}

print_normal_split() {
    local j
    for j in "${!length_success_array[@]}"; do
        if [ $j -eq 0 ]; then
            echo -n "|"
        else
            echo -n "+"
        fi
        print_str_sequence "-" ${length_success_array[j]}
    done
    echo -n "|"
}

print_error_split() {
    local j
    for j in "${!length_error_array[@]}"; do
        if [ $j -eq 0 ]; then
            echo -n "|"
        else
            echo -n "+"
        fi
        print_str_sequence "-" ${length_error_array[j]}
    done
    echo -n "|"
}

print_stat_split(){
    local print_width=$1
    local single_width=$2
    local j
    for (( j=0; j<print_width; j++ )); do
        if [ $j -eq 0 ]; then
            echo -n "|"
        else
            echo -n "+"
        fi
        print_str_sequence "-" $single_width
    done
    echo "|"
}

print_stat_result(){
    local dates="$1"
    local data="$2"
    local print_length=$3
    local iterator=$(echo "$dates" | sed 's/ /\n/g')
    local single_width=12
    local print_width
    if [ $PRINT_STAT_WIDTH -gt $print_length ]; then
        print_width=$print_length
    else
        print_width=$PRINT_STAT_WIDTH
    fi
    # 打印第一行
    local i
    local j
    local k
    for (( i=0; i<print_width; i++ )); do
        echo -n "+"
        print_str_sequence "-" $single_width
    done
    echo "+"
    local for_size=$(( (print_length + print_length - 1) / print_width ))
    for (( i=0; i<for_size; i++ )); do
        for (( j=1; j<=print_width; j++ )); do
            local get_index=$((i * print_width + j))
            local print_item=$(echo "$dates" | awk '{print $'"$get_index"'}')
            print_item=" $print_item "
            echo -n "|${print_item}"
            local white_space_length=$((single_width - $(cal_size "$print_item")))
            print_str_sequence " " $white_space_length
            local next_index=$((get_index + 1))
            if [ $next_index -gt $print_length ]; then
                break
            fi
        done
        echo "|"
        print_stat_split $j $single_width
        for (( j=1; j<=print_width; j++ )); do
            local get_index=$((i * print_width + j))
            local print_item=$(echo "$data" | awk '{print $'"$get_index"'}')
            print_item=" $print_item "
            echo -n "|${print_item}"
            local white_space_length=$((single_width - $(cal_size "$print_item")))
            print_str_sequence " " $white_space_length
            local next_index=$((get_index + 1))
            if [ $next_index -gt $print_length ]; then
                break
            fi
        done
        echo "|"
        # 判断是否最后一行
        if [ $get_index -eq $print_length ]; then
            for ((k=0; k<j; k++)); do
                echo -n "+"
                print_str_sequence "-" $single_width
            done
            echo "+"
        else
            print_stat_split $j $single_width
        fi
    done
    echo
}

print_static_stat_count() {
    local count=$1
    local print_width=12
    local print_item=" $count "
    local count_length=$(cal_size "$print_item")
    if [ $print_width -lt $count_length ]; then
        print_width=$count_length
    fi
    echo -n "+"
    print_str_sequence "-" $print_width
    echo "+"
    local print_title=" 总量 "
    echo -n "|$print_title"
    local white_space_length=$((print_width - $(cal_size "$print_title")))
    print_str_sequence " " $white_space_length
    echo "|"
    echo -n "|"
    print_str_sequence "-" $print_width
    echo "|"
    echo -n "|$print_item"
    white_space_length=$((print_width - $(cal_size "$print_item")))
    print_str_sequence " " $white_space_length
    echo "|"
    echo -n "+"
    print_str_sequence "-" $print_width
    echo "+"

}

print_normal_message(){
    local check_item=$1
    local normal_message=$2
    echo
    echo -n "|$check_item"
    local white_space_length=$((length_success_array[0] - $(cal_size "$check_item")))
    print_str_sequence " " $white_space_length
    echo -n ""
    #-------以下为按照列的宽度切割normal_message
    # 使用i来遍历normal_message字符串的每个索引
    local i
    local half_length=$((length_success_array[1] / 2))
    # 每次切割后的字符串,考虑到中文字符所占宽度是英文字符的两倍，先取1/2宽度的字符
    local sub_str="${normal_message:0:$half_length}"
    for ((i=$half_length; i < ${#normal_message}; )); do
        local char="${normal_message:$i:1}"
        if [[ $(cal_size "$sub_str$char") -le ${length_success_array[1]} ]]; then
            sub_str="$sub_str$char"
            ((i++))
        else
            # 切割完成的字符串sub_str
            echo -n "|$sub_str"
            white_space_length=$((length_success_array[1] - $(cal_size "$sub_str")))
            print_str_sequence " " $white_space_length
            echo "|"
            # normal_message未打印完，打印下一行
            echo -n "|"
            print_str_sequence " " ${length_success_array[0]}
            sub_str=" ${normal_message:$i:$half_length}"
            i=$((i + half_length))
        fi
    done
    # 打印最后一个sub_str
    echo -n "|$sub_str"
    white_space_length=$((length_success_array[1] - $(cal_size "$sub_str")))
    print_str_sequence " " $white_space_length
    echo "|"
}

check_server_online(){
    local ip=$1
    local server=$2
    if [[ "$is_lower_version" == "false" ]]; then
        local isRunning=$(ssh -n $ip "dbserver.sh status_server $server" | grep "ALL SERVICES STOPPED" | wc -l)
        if [ $isRunning -eq 0 ]; then
            echo "Running"
        else
            echo "Stopped"
        fi
    else
        local isRunning=$(ssh -n $ip '
            status=$(dbserver.sh status_server '$server')
            if [ $? -eq 0 ]; then
	            if [ -n "$status" ]; then
		            if grep -q "Running" <<< "$status"; then
			            echo "Running"
		            elif grep -q "Stopped" <<< "$status"; then
			            echo "Stopped"
		            fi
	            else
		            echo "Running"
	            fi
            else
	            echo "Stopped"
            fi
        ')
        echo "$isRunning"
    fi
}

check_server_online_all(){
    local server="$1"
    if [[ "$is_lower_version" == "false" ]]; then
        local isRunning=$(ssh -n $ip "dbserver.sh status_server_all $server" | grep "ALL SERVICES STOPPED" | wc -l)
        if [ $isRunning -eq 0 ]; then
            echo "Running"
        else
            echo "Stopped"
        fi
    else
        local ip
        local status
        local isRunning=$(dbserver.sh status | grep -E "$server" | grep "Stopped" | wc -l)
        if [ $isRunning -eq 0 ]; then
            echo "Running"
        else
            echo "Stopped"
        fi
    fi
}

check_hdfs_server_all_status(){
    if [[ "$is_lower_version" == "false" ]]; then
        check_server_online_all "hdfs"
    else
        check_server_online_all "hadoop NameNode|hadoop DataNode|hadoop JournalNode|hadoop SecondaryNameNode|DFSZKFailoverController"
    fi

}

check_hbase_server_all_status(){
    if [[ "$is_lower_version" == "false" ]]; then
        check_server_online_all "hbase"
    else
        check_server_online_all "Hbase HMaster|hbase HRegionServer"
    fi
}

check_redis_server_all_status(){
    if [[ "$is_lower_version" == "false" ]]; then
        check_server_online_all "Redis"
    else
        check_server_online_all "redis-server"
    fi
}

check_pg_server_all_status(){
    if [[ "$is_lower_version" == "false" ]]; then
        check_server_online_all "pg"
    else
        local pid=`ps -elf | grep "postmaster" | grep -v grep | grep " -p 5432" | awk '{ print $4 }'`
        if [ -z "$pid" ]; then
            echo "Stopped"
        else
            echo "Running"
        fi
    fi
}

print_error_message(){
    local check_item=$1
    local error_message=$2
    local deal_method=$3
    echo
    echo -n "|$check_item"
    local white_space_length=$((length_error_array[0] - $(cal_size "$check_item")))
    print_str_sequence " " $white_space_length
    echo -n ""
    #-------以下为按照列的宽度切割error_message
    # 使用i来遍历error_message字符串的每个索引
    local i
    local half_length=$((length_error_array[1] / 2))
    local print_deal=0
    # 每次切割后的字符串,考虑到中文字符所占宽度是英文字符的两倍，先取1/2宽度的字符
    local sub_str="${error_message:0:$half_length}"
    for ((i=$half_length; i < ${#error_message}; )); do
        local char="${error_message:$i:1}"
        if [[ $(cal_size "$sub_str$char") -le ${length_error_array[1]} ]]; then
            sub_str="$sub_str$char"
            ((i++))
        else
            # 切割完成的字符串sub_str
            echo -n "|$sub_str"
            white_space_length=$((length_error_array[1] - $(cal_size "$sub_str")))
            print_str_sequence " " $white_space_length
            if [ $print_deal -eq 0 ]; then
                print_deal=1
            else
                deal_method=""
            fi
            echo -n "|$deal_method"
            white_space_length=$((length_error_array[2] - $(cal_size "$deal_method")))
            print_str_sequence " " $white_space_length
            echo "|"
            # error_message未打印完，打印下一行
            echo -n "|"
            print_str_sequence " " ${length_error_array[0]}
            sub_str=" ${error_message:$i:$half_length}"
            i=$((i + half_length))
        fi
    done
    # 打印最后一个sub_str
    echo -n "|$sub_str"
    white_space_length=$((length_error_array[1] - $(cal_size "$sub_str")))
    print_str_sequence " " $white_space_length
    if [ $print_deal -ne 0 ]; then
        deal_method=""
    fi
    echo -n "|$deal_method"
    white_space_length=$((length_error_array[2] - $(cal_size "$deal_method")))
    print_str_sequence " " $white_space_length
    echo "|"
}

check_node_online_status(){
    local line
    local online_node=""
    local offline_node=""
    dbserver.sh stop_server_all daemon > /dev/null 2>&1
    local status_detail=$(dbserver.sh status_detail)
    status_detail=$(echo "$status_detail" | sed -r "s/\x1B\[[0-9;]*[mGKF]//g")
    local status=$(dbserver.sh status | sed -r "s/\x1B\[[0-9;]*[mGKF]//g")
    if [[ "$is_lower_version" == "false" ]]; then
        local command_result=$(echo "${status_detail}" | grep -E  "([0-9]{1,3}\.){3}[0-9]{1,3}")
        while IFS= read -r line; do
            if [[ "$line" =~ "[OFFLINE]" ]]; then
                local node=$(echo "$line" | awk '{print $1}')
                offline_node="${offline_node}${node},"
            else
                local node=$(echo "$line" | awk '{print $(NF-2)}')
                online_node="${online_node}${node},"
            fi
        done <<< "$command_result"
    else
        local command_result=$(echo "${status}" | grep -E  "([0-9]{1,3}\.){3}[0-9]{1,3}")
        while IFS= read -r line; do
            local ip=$(echo "$line" | grep -E -o "([0-9]{1,3}\.){3}[0-9]{1,3}")
            if grep -q "is not online" <<< "$line"; then
                offline_node="${offline_node}${ip},"
            else
                online_node="${online_node}${ip},"
            fi
        done <<< "${command_result}"

    fi
    online_node="${online_node%,}"
    offline_node="${offline_node%,}"
    if [ -n "$online_node" ]; then
        #打印正常信息
        local normal_message=$(printf "${NODE_ONLINE_STATUS[1]}" "$online_node")
        print_normal_message "${NODE_ONLINE_STATUS[0]}" "$normal_message"
        print_normal_split
    fi
    if [ -n "$offline_node" ]; then
        #缓存异常信息
        local error_message=$(printf "${NODE_ONLINE_STATUS[2]}" "$offline_node")
        print_error_message "${NODE_ONLINE_STATUS[0]}" "$error_message" "${NODE_ONLINE_STATUS[3]}" >> $OUTPUT_FILE_ERROR_INFO_TEMP
        print_error_split >> $OUTPUT_FILE_ERROR_INFO_TEMP
    fi

    local node
    local all_running_host=""
    declare -A stop_server_map # stop_server -> node
    local line
    if [[ "$is_lower_version" == "false" ]]; then
        while IFS= read -r line; do
            local read_ip=$(echo "$line" | grep -E -o "([0-9]{1,3}\.){3}[0-9]{1,3}")
            if [[ -n "$read_ip" ]]; then
                node=$(echo "$line" | awk '{print $(NF-2)}')
            elif [[ $line == STOPPED:* ]]; then
                stop_server=$(echo $line | sed -e 's/STOPPED: //' -e 's/\[\|\]//g' -e 's/Daemon//g' -e 's/NONE//g' | xargs -n1 | sed 's/^[[:space:]]*//;s/[[:space:]]*$//' | paste -sd ",")
                if [[ -z "$stop_server" ]]; then
                    all_running_host="${all_running_host}${node},"
                else
                    if [[ -z "${stop_server_map["$stop_server"]}" ]]; then
                        stop_server_map["$stop_server"]="$node"
                    else
                        stop_server_map["$stop_server"]="${stop_server_map["$stop_server"]},$node"
                    fi
                fi
            fi
        done <<< "$status_detail"
    else
        local stopped_server_grep=$(echo "$status" | grep -E  "([0-9]{1,3}\.){3}[0-9]{1,3}|Stopped")
        local stop_server=""
        while IFS= read -r line; do
            local read_ip=$(echo "$line" | grep -E -o "([0-9]{1,3}\.){3}[0-9]{1,3}")
            if [ -n "$read_ip" ]; then
                if [ -n "${stop_server}" ]; then
                    stop_server="${stop_server%,}"
                    if [[ -z "${stop_server_map["$stop_server"]}" ]]; then
                        stop_server_map["$stop_server"]="$node"
                    else
                        stop_server_map["$stop_server"]="${stop_server_map["$stop_server"]},$node"
                    fi
                elif [ -n "$node" ]; then
                    all_running_host="${all_running_host}${node},"
                fi
                stop_server=""
                node=""
                if grep -q "is not online" <<< "$line"; then
                    continue
                fi
                node=$(ssh -n ${read_ip} "cat ${CFGCHG_GLOBAL} | grep devCode" | awk -F '=' '{print $2}')
            else
                local current_stop=$(echo "$line" | awk '{print $2}')
                stop_server="${stop_server}${current_stop},"
            fi
        done <<< "$stopped_server_grep"
        if [ -n "${stop_server}" ]; then
            stop_server="${stop_server%,}"
            if [[ -z "${stop_server_map["$stop_server"]}" ]]; then
                stop_server_map["$stop_server"]="$node"
            else
                stop_server_map["$stop_server"]="${stop_server_map["$stop_server"]},$node"
            fi
        elif [ -n "$node" ]; then
            all_running_host="${all_running_host}${node},"
        fi
    fi
    all_running_host="${all_running_host%,}"
    if [ -n "$all_running_host" ]; then
        # 打印正常信息
        local normal_message=$(printf "${SERVER_RUNNING_STATUS[1]}" "$all_running_host")
        print_normal_message "${SERVER_RUNNING_STATUS[0]}" "$normal_message"
        print_normal_split
    fi
    local servers
    for servers in "${!stop_server_map[@]}"; do
        local nodes="${stop_server_map[$servers]}"
        #缓存异常信息
        local error_message=$(printf "${SERVER_RUNNING_STATUS[2]}" "$nodes" "$servers")
        print_error_message "${SERVER_RUNNING_STATUS[0]}" "$error_message" "${SERVER_RUNNING_STATUS[3]}" >> $OUTPUT_FILE_ERROR_INFO_TEMP
        print_error_split >> $OUTPUT_FILE_ERROR_INFO_TEMP
    done
}

check_es_index_status(){
    # 判断SLAVE_IP的es服务是否正常
    if [[ "$(check_server_online ${SLAVE_IP} Elasticsearch)" == "Stopped" ]]; then
        return
    fi
    local es_index_status=$(curl -X GET -s http://${SLAVE_IP}:9200/_cat/indices?v 2>/dev/null)
    printf "【ES 索引信息】：\n# curl -X GET -s http://${SLAVE_IP}:9200/_cat/indices?v\n${es_index_status}\n" >> $OUTPUT_FILE_ES
    es_index_status=$(echo "$es_index_status" | sed '/^$/d' | grep -v "docs.count")
    local red_index=""
    local yellow_index=""
    local line
    while IFS= read -r line; do
        local health=$(echo "$line" | awk '{print $1}')
        local index=$(echo "$line" | awk '{print $3}')
        if [[ "$health" == "yellow" ]]; then
            yellow_index="${yellow_index}${index},"
        elif [[ "$health" == "red" ]]; then
            red_index="${red_index}${index},"
        fi
    done <<< "$es_index_status"
    red_index="${red_index%,}"
    yellow_index="${yellow_index%,}"
    #拼接异常信息
    local error_message=""
    if [[ -n "$yellow_index" ]]; then
        local yellow=$(printf "${ES_INDEX_STATUS[2]}" "yellow" "$yellow_index")
        error_message="${error_message}${yellow};"
    fi
    if [[ -n "$red_index" ]]; then
        local red=$(printf "${ES_INDEX_STATUS[2]}" "red" "$red_index")
        error_message="${error_message}${red};"
    fi
    if [[ -n "$error_message" ]]; then
        error_message="${error_message%;}"
        print_error_message "${ES_INDEX_STATUS[0]}" " $error_message " "${ES_INDEX_STATUS[3]}" >> $OUTPUT_FILE_ERROR_INFO_TEMP
        print_error_split >> $OUTPUT_FILE_ERROR_INFO_TEMP
    else
        print_normal_message "${ES_INDEX_STATUS[0]}" "${ES_INDEX_STATUS[1]}"
        print_normal_split
    fi
    local read_only_index=""
    while IFS= read -r line; do
        local index_name=$(echo "$line" | awk '{print $3}')
        local index_setting=$(curl -s -X  GET http://${SLAVE_IP}:9200/${index_name}/_settings)
        printf "【${index_name}索引配置参数】：\n# curl -s -X  GET http://${SLAVE_IP}:9200/${index_name}/_settings\n${index_setting}\n" >> $OUTPUT_FILE_ES
        if grep -qE '"read_only_allow_delete":"true"|"read_only":"true"' <<< "$index_setting"; then
            read_only_index="${read_only_index}${index_name},"
        fi
    done <<< "$es_index_status"
    read_only_index="${read_only_index%,}"
    if [[ -n "$read_only_index" ]]; then
        local error_message=$(printf "${ES_INDEX_RO_STATUS[2]}" "$read_only_index")
        print_error_message "${ES_INDEX_RO_STATUS[0]}" "$error_message" "${ES_INDEX_RO_STATUS[3]}" >> $OUTPUT_FILE_ERROR_INFO_TEMP
        print_error_split >> $OUTPUT_FILE_ERROR_INFO_TEMP
    else
        print_normal_message "${ES_INDEX_RO_STATUS[0]}" "${ES_INDEX_RO_STATUS[1]}"
        print_normal_split
    fi
}

check_es_cluster_status(){
    # 判断SLAVE_IP的es服务是否正常
    if [[ "$(check_server_online ${SLAVE_IP} Elasticsearch)" == "Stopped" ]]; then
        return
    fi
    local cluster_health=$(curl -X GET -s http://${SLAVE_IP}:9200/_cluster/health?pretty 2>/dev/null)
    printf "【ES 索引状态】：\n# curl -X GET -s http://${SLAVE_IP}:9200/_cluster/health?pretty\n${cluster_health}\n" >> $OUTPUT_FILE_ES
    local status=$(echo "$cluster_health" | grep '^  \"status\" ' | awk '{print $3}' | sed 's/,//g' | sed "s/\"//g")
    if [[ "$status" == "green" ]]; then
        print_normal_message "${ES_CLUSTER_STATUS[0]}" "${ES_CLUSTER_STATUS[1]}"
        print_normal_split
    else
        local error_message=$(printf "${ES_CLUSTER_STATUS[2]}" "$status")
        print_error_message "${ES_CLUSTER_STATUS[0]}" "$error_message" "${ES_CLUSTER_STATUS[3]}" >> $OUTPUT_FILE_ERROR_INFO_TEMP
        print_error_split >> $OUTPUT_FILE_ERROR_INFO_TEMP
    fi
}

check_es_shard_status(){
    # 判断SLAVE_IP的es服务是否正常
    if [[ "$(check_server_online ${SLAVE_IP} Elasticsearch)" == "Stopped" ]]; then
        return
    fi
    local es_shards_info=$(curl -XGET http://${SLAVE_IP}:9200/_cat/shards?v 2>/dev/null)
    printf "【ES shards信息】：\n# curl -XGET http://${SLAVE_IP}:9200/_cat/shards?v\n${es_shards_info}\n" >> $OUTPUT_FILE_ES
    es_shards_info=$(echo "$es_shards_info" | sed '1d')
    local shards_total=$(echo "$es_shards_info" | wc -l)
    local shard_active=$(echo "$es_shards_info" | awk '{print $4}' | grep -E "^STARTED$" | wc -l)

    if [[ "$shards_total" == "$shard_active" ]]; then
        local normal_message=$(printf "${ES_SHARD_STATUS[1]}" $shards_total $shard_active)
        print_normal_message "${ES_SHARD_STATUS[0]}" "$normal_message"
        print_normal_split
    else
        local error_message=$(printf "${ES_SHARD_STATUS[2]}" $shards_total $shard_active)
        print_error_message "${ES_SHARD_STATUS[0]}" "$error_message" "${ES_SHARD_STATUS[3]}" >> $OUTPUT_FILE_ERROR_INFO_TEMP
        print_error_split >> $OUTPUT_FILE_ERROR_INFO_TEMP
    fi
}

check_hdfs_corrupt_block_status(){
    # 判断hdfs服务是否正常
    if [[ "$(check_hdfs_server_all_status)" == "Stopped" ]]; then
        return
    fi
    local hdfs_info=$(hdfs fsck / 2>/dev/null)
    printf "【HDFS状态】：\n# hdfs fsck /\n" >> $OUTPUT_FILE_HDFS
    echo "$hdfs_info" >> $OUTPUT_FILE_HDFS
    local status=$(echo "$hdfs_info" | grep "^The filesystem under path" | awk '{print $NF}')
    if [[ "$status" == "HEALTHY" ]]; then
        local normal_message=$(printf "${HDFS_COMPONENTS_STATUS[1]}" "$status")
        print_normal_message "${HDFS_COMPONENTS_STATUS[0]}" "$normal_message"
        print_normal_split
    else
        local error_message=$(printf "${HDFS_COMPONENTS_STATUS[2]}" "$status")
        print_error_message "${HDFS_COMPONENTS_STATUS[0]}" "$error_message" "${HDFS_COMPONENTS_STATUS[3]}" >> $OUTPUT_FILE_ERROR_INFO_TEMP
        print_error_split >> $OUTPUT_FILE_ERROR_INFO_TEMP
    fi
    local corrupt_blocks=$(echo "$hdfs_info" | grep "^ Corrupt blocks" | awk '{print $3}')
    if [[ "$corrupt_blocks" == "0" ]]; then
        print_normal_message "${HDFS_CORRUPT_BLOCK_STATUS[0]}" "${HDFS_CORRUPT_BLOCK_STATUS[1]}"
        print_normal_split
    else
        local error_message=$(printf "${HDFS_CORRUPT_BLOCK_STATUS[2]}" "$corrupt_blocks")
        print_error_message "${HDFS_CORRUPT_BLOCK_STATUS[0]}" "$error_message" "${HDFS_CORRUPT_BLOCK_STATUS[3]}" >> $OUTPUT_FILE_ERROR_INFO_TEMP
        print_error_split >> $OUTPUT_FILE_ERROR_INFO_TEMP
    fi

}

check_hbase_components_status(){
    # 判断hbase服务是否正常
    if [[ "$(check_hbase_server_all_status)" == "Stopped" ]]; then
        return
    fi
    local hbase_status=$(hbase hbck 2>/dev/null)
    printf "【HBase组件状态】：\n# hbase hbck\n${hbase_status}\n" >> $OUTPUT_FILE_HBASE
    # 检查输出中是否包含"Status: OK"字符串
    local status=$(echo "$hbase_status" | grep "^Status" | awk '{print $NF}')
    if [[ "OK" == "$status" ]]; then
        local normal_message=$(printf "${HBASE_COMPONENTS_STATUS[1]}" "$status")
        print_normal_message "${HBASE_COMPONENTS_STATUS[0]}" "$normal_message"
        print_normal_split
    else
        local error_message
        if [[ -n "$status" ]]; then
            error_message=$(printf "${HBASE_COMPONENTS_STATUS[2]}" "$status")
        else
            error_message=$(printf "${HBASE_COMPONENTS_STATUS[2]}" "执行hbase hbck报错")
        fi
        print_error_message "${HBASE_COMPONENTS_STATUS[0]}" "$error_message" "${HBASE_COMPONENTS_STATUS[3]}" >> $OUTPUT_FILE_ERROR_INFO_TEMP
        print_error_split >> $OUTPUT_FILE_ERROR_INFO_TEMP
    fi
}

check_hbase_cluster_load_status(){
    # 判断hbase服务是否正常
    if [[ "$(check_hbase_server_all_status)" == "Stopped" ]]; then
        return
    fi
    local tip="正在检查HBASE负载，预计耗时1-2分钟..."
    echo -n "$tip"
    local hbase_load_info=$(hbase shell <<< "status 'summary'" 2>/dev/null)
    printf "【HBase集群状态】：\n# hbase shell <<< \"status 'summary'\"\n${hbase_load_info}\n" >> $OUTPUT_FILE_HBASE
    local hbase_avg_load=$(echo "$hbase_load_info" | grep "average load" | awk -F ',' '{print $NF}' | awk '{print $1}')
    local hbase_load_detail=$(hbase shell <<< "status 'detailed'" 2>/dev/null)
    printf "【HBase集群详情】：\n# hbase shell <<< \"status 'detailed'\"\n${hbase_load_detail}\n" >> $OUTPUT_FILE_HBASE
    local hbase_total_load=$(echo "$hbase_load_detail" | grep "numberOfOnlineRegions" | tr ',' '\n' | grep "numberOfOnlineRegions" | awk -F '=' '{print $2}' | awk 'BEGIN{sum=0}{sum+=$1}END{print sum}')
    local normal_message=$(printf "${HBASE_CLUSTER_LOAD_STATUS[1]}" "$hbase_avg_load" "$hbase_total_load")
    local move_size=$((CURRENT_WINDOW_SIZE - 1))
    echo -ne "\033[F\033[${move_size}C"
    print_normal_message "${HBASE_CLUSTER_LOAD_STATUS[0]}" "$normal_message"
    print_normal_split
}

check_greenplum_components_status(){
    # 判断gp服务是否正常
    local has_greenplum=$(query_server_status | grep "GreenPlum")
    if [ -z "$has_greenplum" ]; then
        return
    fi
    if [[ "$(check_server_online_all GreenPlum)" == "Stopped" ]]; then
        return
    fi
    local result=$(su gpadmin -c "$GP_PSQL -p 6543 -d postgres --command 'select * from gp_segment_configuration'" 2>/dev/null)
    printf "【GP 各segment的状态】：\n# su gpadmin -c \"$GP_PSQL -p 6543 -d postgres --command 'select * from gp_segment_configuration'\"\n${result}\n" >> $OUTPUT_FILE_GP
    local head=$(echo "$result" | grep "status" | head -n 1)
    local del_str=${head%%status*}
    local index=${#del_str}
    local status=$(echo "$result" | sed '1,2d' | grep -v "^(" | sed '/^$/d')
    status=$(echo "$status" | sed -n "s/^\(.\{$index\}\)\(.\).*/\2/p")
    local total=$(echo "$status" | wc -l)
    local up=$(echo "$status" | grep "u" | wc -l)
    local error_message
    if [[ $total -eq 0 ]]; then
        error_message=" 无法获取到segment状态 "
    elif [[ $total -ne $up ]]; then
        error_message=$(printf "${GREENPLUM_COMPONENTS_STATUS[1]}" $total $up)
    else
        local normal_message=$(printf "${GREENPLUM_COMPONENTS_STATUS[1]}" $total $up)
        print_normal_message "${GREENPLUM_COMPONENTS_STATUS[0]}" "$normal_message"
        print_normal_split
    fi
    if [[ -n "$error_message" ]]; then
        print_error_message "${GREENPLUM_COMPONENTS_STATUS[0]}" "$error_message" "${GREENPLUM_COMPONENTS_STATUS[3]}" >> $OUTPUT_FILE_ERROR_INFO_TEMP
        print_error_split >> $OUTPUT_FILE_ERROR_INFO_TEMP
    fi
}

check_redis_components_status(){
    # 判断redis服务是否正常
    if [[ "$(check_redis_server_all_status)" == "Stopped" ]]; then
        return
    fi
    # 老版本的查询redis组件命令不同
    local redis_version=$(/bigdata/salut/components/redis/bin/redis-cli --version)
    printf "【REDIS版本】：\n# /bigdata/salut/components/redis/bin/redis-cli --version\n${redis_version}\n" >> $OUTPUT_FILE_REDIS
    redis_version=$(echo "$redis_version" | awk '{print $2}' | awk -F '.' '{print $1}')
    local port_9000_exist=$(telnet ${SLAVE_IP} 9000 2>&1 | grep "Connection refused")
    local port
    if [ -n "${port_9000_exist}" ]; then
        port=9001
    else
        port=9000
    fi
    if (( redis_version < 5 )); then
        local cluster_info=$(/bigdata/salut/components/redis/bin/redis-cli -a admin_123 -h $SLAVE_IP -p ${port} -c <<< "CLUSTER INFO")
        local offline_redis=$(/bigdata/salut/components/redis/bin/redis-cli -a admin_123 -h $SLAVE_IP -p ${port} -c <<< "CLUSTER NODES")
        printf "【REDIS集群信息】：\n# /bigdata/salut/components/redis/bin/redis-cli -a admin_123 -h $SLAVE_IP -p ${port} -c <<< \"CLUSTER INFO\"\n${cluster_info}\n" >> $OUTPUT_FILE_REDIS
        printf "【REDIS节点信息】：\n# /bigdata/salut/components/redis/bin/redis-cli -a admin_123 -h $SLAVE_IP -p ${port} -c <<< \"CLUSTER NODES\"\n${offline_redis}\n" >> $OUTPUT_FILE_REDIS
        cluster_info=$(echo "$cluster_info" | grep "cluster_state" | awk -F ':' '{print $2}' | sed 's/\r//g')
        offline_redis=$(echo "$offline_redis" | grep "disconnected" | wc -l)
        local is_normal=$([[ "$cluster_info" == "ok" && "$offline_redis" == "0" ]] && echo "true" || echo "false")
    else
        local redis_info=$(redis-cli -a admin_123 -c --cluster check $SLAVE_IP:${port} 2>/dev/null)
        printf "【REDIS集群信息】：\n# redis-cli -a admin_123 -c --cluster check $SLAVE_IP:${port}\n${redis_info}\n" >> $OUTPUT_FILE_REDIS
        local status=$(echo "$redis_info" | grep "[OK]" | wc -l)
        local is_normal=$([[ "$status" == "3" ]] && echo "true" || echo "false")
    fi
    if [[ "$is_normal" == "true" ]]; then
        print_normal_message "${REDIS_COMPONENTS_STATUS[0]}" "${REDIS_COMPONENTS_STATUS[1]}"
        print_normal_split
    else
        print_error_message "${REDIS_COMPONENTS_STATUS[0]}" "${REDIS_COMPONENTS_STATUS[2]}" "${REDIS_COMPONENTS_STATUS[3]}" >> $OUTPUT_FILE_ERROR_INFO_TEMP
        print_error_split >> $OUTPUT_FILE_ERROR_INFO_TEMP
    fi
}

check_consumer_group_status(){
    # 判断kafka服务是否正常
    if [[ "$(check_server_online_all Kafka)" == "Stopped" ]]; then
        return
    fi

    print_log "【appOffsetChecker.sh】" >> $OUTPUT_FILE_KAFKA
    local offset_checker_result=$(appOffsetChecker.sh 2>/dev/null | tee -a $OUTPUT_FILE_KAFKA)
    local current_groups=$(echo "$offset_checker_result" | grep -v "Usage" | grep -v "Here are")
    local group
    # 从flume-conf.properties文件中获取消费者组列表
    if [ -f "$FLUME_CONF_PROPERTIES" ]; then
        local channels=$(cat "$FLUME_CONF_PROPERTIES" | grep "^agent.channels=" | awk -F '=' '{print $2}' | sed 's/ /\n/g' | sed '/^$/d')
        local channel
        local lack_groups=""
        while IFS= read -r channel; do
            local prefix=$(cat "$FLUME_CONF_PROPERTIES" | grep "^agent.channels.$channel.kafka.consumer.group.id=" | awk -F '=' '{print $2}')
            local topic=$(cat "$FLUME_CONF_PROPERTIES" | grep "^agent.channels.$channel.kafka.topic=" | awk -F '=' '{print $2}')
            if [ -n "$prefix" ] || [ -n "$topic" ]; then
                local group="$prefix-$topic"
                if ! grep -qx "$group" <<< "$current_groups"; then
                    lack_groups="$lack_groups$group,"
                fi
            fi
        done <<< "$channels"
        lack_groups="${lack_groups%,}"
        if [[ -z "$lack_groups" ]]; then
            print_normal_message "${CONSUMER_GROUP_STATUS[0]}" "${CONSUMER_GROUP_STATUS[1]}"
            print_normal_split
        else
            local error_message=$(printf "${CONSUMER_GROUP_STATUS[2]}" "$lack_groups")
            print_error_message "${CONSUMER_GROUP_STATUS[0]}" "$error_message" "${CONSUMER_GROUP_STATUS[3]}" >> $OUTPUT_FILE_ERROR_INFO_TEMP
            print_error_split >> $OUTPUT_FILE_ERROR_INFO_TEMP
        fi
    fi

    if grep -q "Here are consumer group available" <<< "$offset_checker_result"; then
        # 校验消费情况
        # 这里比较耗时，打印耗时提醒
        local tip="正在检查消费，预计耗时5-15分钟..."
        echo -n "$tip"
        local check_group
        declare -A consume_current
        local next_check_groups=""
        while IFS= read -r group; do
            local group_consume=$(appOffsetChecker.sh "$group" 2>/dev/null)
            printf "【$(date +'%Y-%m-%d %H:%M:%S')】【${group}消费情况】\n# appOffsetChecker.sh \"$group\"\n${group_consume}\n" >> $OUTPUT_FILE_KAFKA
            if [ $(echo "$group_consume" | wc -l) -gt 2 ]; then
                local lag
                local current
                if grep -q "CURRENT=" <<< "$group_consume"; then
                    lag=$(echo "$group_consume" | tail -n 1 | awk '{print $3}' | awk -F '=' '{print $2}')
                    current=$(echo "$group_consume" | tail -n 1 | awk '{print $1}' | awk -F '=' '{print $2}')
                else
                    lag=$(echo "$group_consume" | sed '/^$/d' | grep -v "TOPIC" | awk '{s1+=$5}END{print s1}')
                    current=$(echo "$group_consume" | sed '/^$/d' | grep -v "TOPIC" | awk '{s2+=$3}END{print s2}')
                fi
                if [[ "$lag" != "0" ]]; then
                    consume_current["$group"]="$current"
                fi
            fi
        done <<< "$current_groups"

        if [ ${#consume_current[@]} -ne 0 ]; then
            # 睡眠6分钟再校验下次消费
            sleep 360
        fi
        local stop_groups=""
        for group in "${!consume_current[@]}"; do
            old_current="${consume_current[$group]}"
            group_consume=$(appOffsetChecker.sh "$group" 2>/dev/null)
            printf "【$(date +'%Y-%m-%d %H:%M:%S')】【${group}消费情况】\n# appOffsetChecker.sh \"$group\"\n${group_consume}\n" >> $OUTPUT_FILE_KAFKA
            if grep -q "CURRENT=" <<< "$group_consume"; then
                current=$(echo "$group_consume" | tail -n 1 | awk '{print $1}' | awk -F '=' '{print $2}')
            else
                current=$(echo "$group_consume" | sed '/^$/d' | grep -v "TOPIC" | awk '{s2+=$3}END{print s2}')
            fi
            if (( old_current >= current )); then
                stop_groups="$stop_groups$group,"
            fi
        done
        stop_groups="${stop_groups%,}"
        # 校验消费结束，回退打印的tip
        local move_size=$((CURRENT_WINDOW_SIZE - 1))
        echo -ne "\033[F\033[${move_size}C"

        if [[ -z "$stop_groups" ]]; then
            print_normal_message "${FLUME_CONSUMER_STATUS[0]}" "${FLUME_CONSUMER_STATUS[1]}"
            print_normal_split
        else
            local error_message=$(printf "${FLUME_CONSUMER_STATUS[2]}" "$stop_groups")
            print_error_message "${FLUME_CONSUMER_STATUS[0]}" "$error_message" "${FLUME_CONSUMER_STATUS[3]}" >> $OUTPUT_FILE_ERROR_INFO_TEMP
            print_error_split >> $OUTPUT_FILE_ERROR_INFO_TEMP
        fi
    fi
            # 低版本判断消费者组（暂时不实现）
#        local consumer_info=$(echo "$offset_checker_result" | sed -n '/topic/,$p' | grep -v "^topic" | awk '
#        {
#            group = $1          # 第一列为分组键
#            col4 = $4           # 第四列的值
#            col5 = $5           # 第五列的值
#
#            sum_col4[group] += col4  # 累加第四列的值
#            sum_col5[group] += col5  # 累加第五列的值
#        }
#        END {
#            for (g in sum_col4) {
#                print g, sum_col4[g], sum_col5[g]  # 输出分组及累加结果
#            }
#        }
#        ')
#        local line
#        local second_check_topic=""
#        while IFS= read -r line; do
#            local latest=$(echo "$line" | awk '{print $2}')
#            local total=$(echo "$line" | awk '{print $3}')
#            local topic=$(echo "$line" | awk '{print $1}')
#            if [[ "$latest" != "$total" ]]; then
#                consume_current["$topic"]="$latest"
#                second_check_topic="${second_check_topic}^${topic} |"
#            fi
#        done <<< "$consumer_info"
#        if [ ${#consume_current[@]} -ne 0 ]; then
#            # 睡眠6分钟再校验下次消费
#            sleep 360
#        fi
#        print_log "【appOffsetChecker.sh】" >> $OUTPUT_FILE_KAFKA
#        consumer_info=$(appOffsetChecker.sh 2>/dev/null | tee -a $OUTPUT_FILE_KAFKA)
#        consumer_info=$(echo "$consumer_info" | sed -n '/topic/,$p' | grep -E "${second_check_topic}" | awk '
#        {
#            group = $1          # 第一列为分组键
#            col4 = $4           # 第四列的值
#            col5 = $5           # 第五列的值
#
#            sum_col4[group] += col4  # 累加第四列的值
#            sum_col5[group] += col5  # 累加第五列的值
#        }
#        END {
#            for (g in sum_col4) {
#                print g, sum_col4[g], sum_col5[g]  # 输出分组及累加结果
#            }
#        }
#        ')
}

salut_cp(){
    local dest_file=$1
    if which salut-cp >/dev/null 2>&1;then
        salut-cp $dest_file
    else
        local ip
        while IFS= read -r ip; do
            if [[ "$CURRENT_IP" != "$ip" ]]; then
                scp -r $dest_file $ip:`dirname $dest_file`
            fi
        done <<< "$SLAVES"
    fi
}

salut_ssh(){
    if which salut-ssh >/dev/null 2>&1; then
        salut-ssh "$1" "$2"
    else
        local ip
        if [ -n "$1" ]; then
            if [ "$2" == "sync" ]; then
                while IFS= read -r ip; do
                    local remote=$(echo $1 | sed s/@ip/$ip/)
                    local cmd='ssh -n '$ip' '$remote
                    echo $cmd
                    $cmd
                done <<< "$SLAVES"
            else
                while IFS= read -r ip; do
                    {
                        local remote=$(echo $1 | sed s/@ip/$ip/)
                        local cmd='ssh -n '$ip' '$remote
                        echo $cmd
                        $cmd
                    }&
                done <<< "$SLAVES"
                wait
            fi
        fi
    fi
}

check_disk_upper_status(){
    local disk_info=$(salut_ssh "df -h" sync)
    printf "【磁盘占用】：\n# salut-ssh \"df -h\" sync\n" >> $OUTPUT_HARDWARE_INFO
    echo "$disk_info" >> $OUTPUT_HARDWARE_INFO
    # disk_info添加一行，使得结果汇总可以遍历到最后一个循环
    disk_info=$(echo -e "${disk_info}\nssh ip")
    local error_message=""
    local only_read_disks=""
    local line
    local check_ip
    local check_ip_error_message
    local check_ip_only_read
    while IFS= read -r line; do
        if grep -q "^ssh" <<< "$line"; then
            if [ -n "$check_ip_error_message" ]; then
                check_ip_error_message="${check_ip_error_message%,}"
                error_message="${error_message}${check_ip}-->{${check_ip_error_message}},"
            fi
            if [ -n "$check_ip_only_read" ]; then
                check_ip_only_read="${check_ip_only_read%,}"
                only_read_disks="${only_read_disks}${check_ip}-->${check_ip_only_read},"
            fi
            check_ip=$(echo "$line" | awk '{print $2}')
            check_ip_error_message=""
            check_ip_only_read=""
            continue
        fi
        if grep -q "^Filesystem" <<< "$line"; then
            continue
        fi
        local used=$(awk '{print $5}' <<< "$line")
        used="${used%%%}"
        local file_system=$(awk '{print $1}' <<< "$line")
        if (( used >= 75 )); then
            local mounted_on=$(awk '{print $6}' <<< "$line")
            local info="(${mounted_on}=${used}%)"
            check_ip_error_message="$check_ip_error_message$info,"
        fi
        if grep -q "^/" <<< "$file_system"; then
            print_log '【磁盘读写：ssh -n '$check_ip' "blockdev --getro '$file_system'"】' >> $OUTPUT_HARDWARE_INFO
            ssh -n $check_ip "blockdev --getro $file_system 2>&1" >> $OUTPUT_HARDWARE_INFO
            print_separate_rows >> $OUTPUT_HARDWARE_INFO
            local ro=$(ssh -n $check_ip "blockdev --getro $file_system 2>/dev/null")
            if [[ $ro -eq 1 ]]; then
                check_ip_only_read="$check_ip_only_read$file_system,"
            fi
        fi
    done <<< "$disk_info"
    error_message="${error_message%,}"
    only_read_disks="${only_read_disks%,}"
    if [[ -z "$error_message" ]]; then
        print_normal_message "${DISK_UPPER_STATUS[0]}" "${DISK_UPPER_STATUS[1]}"
        print_normal_split
    else
        error_message=$(printf "${DISK_UPPER_STATUS[2]}" "$error_message")
        print_error_message "${DISK_UPPER_STATUS[0]}" "$error_message" "${DISK_UPPER_STATUS[3]}" >> $OUTPUT_FILE_ERROR_INFO_TEMP
        print_error_split >> $OUTPUT_FILE_ERROR_INFO_TEMP
    fi
    if [[ -z "$only_read_disks" ]]; then
        print_normal_message "${DISK_READ_WRITE_STATUS[0]}" "${DISK_READ_WRITE_STATUS[1]}"
        print_normal_split
    else
        error_message=$(printf "${DISK_READ_WRITE_STATUS[2]}" "$error_message")
        print_error_message "${DISK_READ_WRITE_STATUS[0]}" "$error_message" "${DISK_READ_WRITE_STATUS[3]}" >> $OUTPUT_FILE_ERROR_INFO_TEMP
        print_error_split >> $OUTPUT_FILE_ERROR_INFO_TEMP
    fi
}

cal_es_size_in_one_day_range(){
    local core=$1
    local ttl_field=$2
    local date=$3
    local start_ms="$(date -d $date +%s)000"
    local end_ms=$((start_ms + 86400000))
    local result=$(curl -XGET "http://${SLAVE_IP}:9200/${core}*/_count" -H 'Content-Type: application/json' -d"{  \"query\": {    \"range\": {      \"${ttl_field}\": {        \"gte\": ${start_ms},        \"lt\": ${end_ms}      }    }  }}" 2>/dev/null)
    echo -e "【查询索引${core}${date}的数据量】：\n# curl -XGET \"http://${SLAVE_IP}:9200/${core}*/_count" -H 'Content-Type: application/json' -d"{  \"query\": {    \"range\": {      \"${ttl_field}\": {        \"gte\": ${start_ms},        \"lt\": ${end_ms}      }    }  }}\"\n${result}" >> $OUTPUT_FILE_ES
    result=$(echo "$result" | sed 's/,/\n/g' | grep "\"count\"" | awk -F ':' '{print $2}')
    echo $result
}

cal_es_size_in_one_month_range(){
    local core=$1
    local ttl_field=$2
    local date=$3
    local start_ms="$(date -d $date +%s)000"
    local end_ms="$(date -d "$(date -d "$date" +%Y-%m)-01 +1 month" +%s)000"
    local result=$(curl -XGET "http://${SLAVE_IP}:9200/${core}*/_count" -H 'Content-Type: application/json' -d"{  \"query\": {    \"range\": {      \"${ttl_field}\": {        \"gte\": ${start_ms},        \"lt\": ${end_ms}      }    }  }}" 2>/dev/null)
    echo -e "【查询索引${core}$(date -d "$date" +%Y-%m)的数据量】：\n# curl -XGET \"http://${SLAVE_IP}:9200/${core}*/_count" -H 'Content-Type: application/json' -d"{  \"query\": {    \"range\": {      \"${ttl_field}\": {        \"gte\": ${start_ms},        \"lt\": ${end_ms}      }    }  }}\"\n${result}" >> $OUTPUT_FILE_ES
    result=$(echo "$result" | sed 's/,/\n/g' | grep "\"count\"" | awk -F ':' '{print $2}')
    echo $result
}

check_topic_and_ttl_status(){
    # 判断es,pg,hbase服务是否正常
    if [[ "$(check_pg_server_all_status)" == "Stopped" ]]; then
        return
    fi
    local exists_general_data=$(exec_sql "select count(*) from pg_catalog.pg_class where relname = 'tbl_general_data'" | grep -v "count" | grep -v "^-----" | grep -v "(.*row.*)" | sed '/^$/d' | sed 's/ //g')
    if [[ "$exists_general_data" == "0" ]]; then
        # 低版本留存期校验待适配
        return
    fi
    local general_data=$(exec_sql "select * from tbl_general_data;" 2>/dev/null)
    printf "【PG数据库 tbl_general_data 表】：\n# select * from tbl_general_data;\n${general_data}\n" >> $OUTPUT_FILE_TTL
    general_data=$(echo "$general_data" | grep -v "table_name.*unique_id" | grep -v "^----------" | grep -v "(.*row.*)" | sed '/^$/d')
    local status=$(check_server_online_all Kafka)
    if [[ "$status" == "Running" ]]; then
        local current_topics=$(/bigdata/salut/components/kafka/bin/kafka-topics.sh --zookeeper ${ZKCONNECT} --list 2>/dev/null)
        printf "【Kafka 中存在的 topic 信息】：\n# /bigdata/salut/components/kafka/bin/kafka-topics.sh --zookeeper ${ZKCONNECT} --list\n${current_topics}\n" >> $OUTPUT_FILE_KAFKA
        # 校验通用类型
        local general_topics=$(echo "$general_data" | awk -F '|' '{print $1}' | sed 's/ //g')
        local lack_topics=""
        local topic
        while IFS= read -r topic; do
            local auto_topic="auto_$topic"
            if ! grep -qxiE "${topic}|${auto_topic}" <<< "$current_topics"; then
                lack_topics="$lack_topics$topic,"
            fi
        done <<< "$general_topics"

        # 校验非通用类型的topic
        local kafka_create=$(cat /bigdata/salut/conf/kafka/kafkaCreate.sh )
        local not_general_topics=$(echo "$kafka_create" | grep -E 'Names=|Topic=' | awk -F "=" '{print $2}' | sed 's/\"//g' | sed 's/,/\n/g')
        while IFS= read -r topic; do
            if ! grep -qxi "$topic" <<< "$current_topics"; then
                lack_topics="$lack_topics$topic,"
            fi
        done <<< "$not_general_topics"
        # 校验__consumer_offsets
        if ! grep -qx "__consumer_offsets" <<< "$current_topics"; then
            lack_topics="$lack_topics$topic,"
        fi
        lack_topics="${lack_topics%,}"
        if [[ -z "$lack_topics" ]]; then
            print_normal_message "${KAFKA_TOPIC_STATUS[0]}" "${KAFKA_TOPIC_STATUS[1]}"
            print_normal_split
        else
            local error_message=$(printf "${KAFKA_TOPIC_STATUS[2]}" "$lack_topics")
            print_error_message "${KAFKA_TOPIC_STATUS[0]}" "$error_message" "${KAFKA_TOPIC_STATUS[3]}" >> $OUTPUT_FILE_ERROR_INFO_TEMP
            print_error_split >> $OUTPUT_FILE_ERROR_INFO_TEMP
        fi
    fi

    if [[ "$(check_hbase_server_all_status)" == "Stopped" ]]; then
        return
    fi
    if [[ "$(check_server_online_all Elasticsearch)" == "Stopped" ]]; then
        return
    fi
    local tip="正在检查留存期，预计耗时3-5分钟..."
    echo -n "$tip"
    # 查询存在留存期的core htable及对应的留存期天数
    local ttl_config_info=$(exec_sql "SELECT * FROM tbl_ttl_config")
    printf "【PG数据库 tbl_ttl_config 表】：\n# SELECT * FROM tbl_ttl_config\n${ttl_config_info}\n" >> $OUTPUT_FILE_TTL
    ttl_config_info=$(echo "$ttl_config_info" | grep -v "ttl_id.*ttl_code" | grep -v "^------" | grep -v "(.*row.*)" | sed '/^$/d')

    # person_trace的留存期未写在tbl_ttl_config表里，特殊处理
    local person_trace_table=$(echo "$general_data" | grep "person_trace")
    if [ -n "$person_trace_table" ]; then
        local face_image_ttl_value=$(echo "$ttl_config_info" | grep "face_image-" | awk -F '|' '{print $9}' | sed 's/ //g')
        ttl_config_info=$(printf "$ttl_config_info\n%s" " | | | | face_image | person_trace | person_trace | | $face_image_ttl_value | ")
    fi
    local hbase_table=$(echo "$ttl_config_info" | awk -F '|' '{print $6}' | sed 's/ //g' | sed 's/,/\n/g' | sed '/^$/d')
    local ttl_field_values=""
    # 通过判断tbl_ttl_field表是否存在判断是否合入留存期优化
    local is_exists=$(exec_sql "SELECT count(*) FROM pg_tables WHERE tablename = 'tbl_ttl_field'" | grep -v "count" | grep -v "^-----" | grep -v "(.*row.*)" | sed '/^$/d' | sed 's/ //g')
    if [[ "$is_exists" != "0" ]]; then
        ttl_field_values=$(exec_sql "SELECT * FROM tbl_ttl_field")
        printf "【PG数据库 tbl_ttl_field 表】：\n# SELECT * FROM tbl_ttl_field\n${ttl_field_values}\n" >> $OUTPUT_FILE_TTL
        ttl_field_values=$(echo "$ttl_field_values" | grep -v "data_type.*expire_time.*update_time" | grep -v "^---------" | grep -v "(.*row.*)" | sed '/^$/d' )
    fi

    local normal_message=""
    local upper_message=""
    local not_equal_ttl_type=""
    # 遍历每种数据类型，检查实际存储数据天数和配置的留存期
    local line
    while IFS= read -r line; do
        local core_list=$(echo "$line" | awk -F '|' '{print $7}' | sed 's/ //g' | sed 's/,/\n/g')
        if [ -z "$core_list" ]; then
            continue
        fi
        local ttl_value=$(echo "$line" | awk -F '|' '{print $9}' | sed 's/ //g')
        local ttl_name=$(echo "$line" | awk -F '|' '{print $5}' | sed 's/ //g')
        local core
        while IFS= read -r core; do
            local lower_case_table_name=$(echo "$core" | sed 's/-//g')
            local table_name=$(echo "$general_data" | grep -i " $lower_case_table_name " | awk -F '|' '{print $1}' | sed 's/ //g')
            local ttl_field=$(echo "$general_data" | grep -i " $lower_case_table_name " | awk -F '|' '{print $5}' | sed 's/ //g')
            # 判断该索引有没有数据
            local has_count=$(curl -XGET "http://${SLAVE_IP}:9200/${core}*/_count" 2>/dev/null)
            printf "【${core}总数据量】：\n# curl -XGET \"http://${SLAVE_IP}:9200/${core}*/_count\"\n${has_count}\n" >> $OUTPUT_FILE_ES
            has_count=$(echo "$has_count" | grep -v "\"count\":0")
            if [ -z "$has_count" ]; then
                continue
            fi
            if [ -z "$ttl_field" ]; then
                # 适配低版本的es和hbase表
                case "$core" in
                    hrecord | halarm | halarmsync | hrecordsync)
                        ttl_field="pass_time2"
                        ;;
                    mac | virtualmac | rfid)
                        ttl_field="collect_time"
                        ;;
                    videoperson | videobackup)
                        ttl_field="start_time"
                        ;;
                    faceimage)
                        ttl_field="pass_time"
                        ;;
                    staticfacebase)
                        ttl_field="face_birthday"
                        ;;
                    viidmotorvehicle)
                        ttl_field="PassTime"
                        ;;
                esac
            fi

            # 获取留存期清理后的最小ttl_field的值
            local start_time_in_es=$(curl -XGET "http://${SLAVE_IP}:9200/${core}*/_search" -H 'Content-Type: application/json' -d'{  "size": 1,   "sort": [    {      "'${ttl_field}'": {        "order": "asc"      }    }  ]}' 2>/dev/null)
            printf "【查询${core}索引最小的${ttl_field}】：\n# curl -XGET \"http://${SLAVE_IP}:9200/${core}*/_search\" -H 'Content-Type: application/json' -d'{  \"size\": 1,   \"sort\": [    {      "'${ttl_field}'": {        \"order\": \"asc\"      }    }  ]}'\n${start_time_in_es}\n" >> $OUTPUT_FILE_ES
            # 转换成秒级时间戳
            start_time_in_es=$(echo "$start_time_in_es" |  sed -n 's/.*"sort":\[\([^]]*\)].*/\1/p' | sed 's/.\{3\}$//')
            start_time_in_es=$(date -d "$(date -d "@$start_time_in_es" +%Y-%m-%d) 00:00:00" +%s)
            # 转换成毫秒时间戳
            start_time_in_es="${start_time_in_es}000"
            local start_time_in_pg=""
            if [ -n "$ttl_field_values" ]; then
                start_time_in_pg=$(echo "$ttl_field_values" | grep " $lower_case_table_name " | awk -F '|' '{print $2}' | sed 's/ //g')
            fi
            local start_time
            if [ -n "$start_time_in_pg" ]; then
                if (( start_time_in_pg > start_time_in_es )); then
                    start_time=$start_time_in_pg
                else
                    start_time=$start_time_in_es
                fi
            else
                start_time=$start_time_in_es
            fi
            local end_time="$(date -d "tomorrow 00:00:00" +%s)000"
            # 计算end_time-start_time的天数
            local save_days=$(((end_time - start_time) / 86400000))
            local max_ttl=$((ttl_value + 10))
            local difference=$((save_days - ttl_value))
            # 查看hbase留存期是否正确
            if grep -xq " $lower_case_table_name" <<< "$hbase_table"; then
                local hbase_ttl=$(hbase shell <<< "describe '$lower_case_table_name'")
                printf "【HBase ${lower_case_table_name}表留存期】：\n# hbase shell <<< \"describe '$lower_case_table_name'\"\n${hbase_ttl}\n" >> $OUTPUT_FILE_HBASE
                hbase_ttl=$(echo "$hbase_ttl" | sed 's/,/\n/g' | grep "TTL =>" | awk -F '=>' '{print $2}' | awk -F'[()]' '{print $2}' | awk '{print $1}')
                if (( save_days > hbase_ttl )); then
                    not_equal_ttl_type="${not_equal_ttl_type}${lower_case_table_name},"
                fi
                if (( hbase_ttl > max_ttl && save_days < max_ttl )); then
                    upper_message="${upper_message}${lower_case_table_name}=(${hbase_ttl}/${ttl_value});"
                    continue
                fi
            fi

            if (( save_days > max_ttl )); then
                upper_message="${upper_message}${table_name}=(${save_days}/${ttl_value});"
            elif [[ "$ttl_name" == "$table_name" ]]; then
                normal_message="${normal_message}${table_name}=(${save_days}/${ttl_value});"
            fi

            # 统计数据量
            if [[ "$ttl_name" == "$table_name" ]]; then
                local total_count_in_ttl=$(curl -XGET "http://${SLAVE_IP}:9200/${core}*/_count" -H 'Content-Type: application/json' -d"{  \"query\": {    \"range\": {      \"${ttl_field}\": {        \"gte\": $start_time}    }  }}" 2>/dev/null)
                echo -e "【查询索引${core}留存期内的总数】：\n# curl -XGET "http://${SLAVE_IP}:9200/${core}*/_count" -H 'Content-Type: application/json' -d\"{  \"query\": {    \"range\": {      \"${ttl_field}\": {        \"gte\": $start_time}    }  }}\"\n${total_count_in_ttl}" >> $OUTPUT_FILE_ES
                total_count_in_ttl=$(echo "$total_count_in_ttl" | sed 's/,/\n/g' | grep "\"count\"" | awk -F ':' '{print $2}')
                echo -e "\033[1;31m${table_name}数据量统计：【总数：${total_count_in_ttl}】\033[0m\n" >> $OUTPUT_STATISTICS_FILE
                local start_time_s="${start_time%000}"
                local statistics_date=$(get_statistics_date_by_week "$(date -d @$start_time_s +%Y-%m-%d)")
                local week
                echo -e "\033[1;33m近一个月数据量统计：\033[0m" >> $OUTPUT_STATISTICS_FILE
                while IFS= read -r week; do
                    local iterator=$(echo "$week" | sed 's/ /\n/g')
                    local single_day
                    local stat_data=""
                    while IFS= read -r single_day; do
                        local cal_result=$(cal_es_size_in_one_day_range $core $ttl_field $single_day)
                        stat_data="${stat_data}${cal_result} "
                    done <<<"$iterator"
                    stat_data="${stat_data% }"
                    print_stat_result "$week" "$stat_data" $(echo "$iterator" | wc -l) >> $OUTPUT_STATISTICS_FILE
                done <<< "$statistics_date"

                local statistics_month=$(get_statistics_date_by_month "$(date -d @$start_time_s +%Y-%m-%d)")

                echo -e "\033[1;33m近六个月数据量统计：\033[0m" >> $OUTPUT_STATISTICS_FILE
                local month
                while IFS= read -r month; do
                    local iterator=$(echo "$month" | sed 's/ /\n/g')
                    local single_month
                    local stat_data=""
                    local format_months=""
                    while IFS= read -r single_month; do
                        local cal_result=$(cal_es_size_in_one_month_range $core $ttl_field $single_month)
                        stat_data="${stat_data}${cal_result} "
                        local format_month_str=$(date -d "$single_month" +%Y-%m)
                        format_months="${format_months}${format_month_str} "
                    done <<<"$iterator"
                    stat_data="${stat_data% }"
                    print_stat_result "$format_months" "$stat_data" $(echo "$iterator" | wc -l) >> $OUTPUT_STATISTICS_FILE
                done <<< "$statistics_month"
            fi
        done <<< "$core_list"
    done <<< "$ttl_config_info"
    normal_message="${normal_message%;}"
    upper_message="${upper_message%;}"
    not_equal_ttl_type="${not_equal_ttl_type%,}"

    # 统计静态库总量
    local static_count=$(curl -XGET "http://${SLAVE_IP}:9200/static_face_base-*/_count" 2>/dev/null)
    printf "【static_face_base总数据量】：\n# curl -XGET \"http://${SLAVE_IP}:9200/static_face_base-*/_count\"\n${static_count}\n" >> $OUTPUT_FILE_ES
    static_count=$(echo "$static_count" | sed 's/,/\n/g' | grep "\"count\"" | awk -F ':' '{print $2}')
    if [[ "$static_count" != "0" ]]; then
        echo -e "\033[1;31mstatic_face_base数据量统计：\033[0m\n" >> $OUTPUT_STATISTICS_FILE
        print_static_stat_count "$static_count" >> $OUTPUT_STATISTICS_FILE
    fi

    local move_size=$((CURRENT_WINDOW_SIZE - 1))
    echo -ne "\033[F\033[${move_size}C"

    if [ -n "$normal_message" ]; then
        normal_message=$(printf "${TTL_STATUS[1]}" "$normal_message")
        print_normal_message "${TTL_STATUS[0]}" "${normal_message}"
        print_normal_split
    fi
    local error_message=""
    if [ -n "$upper_message" ]; then
        error_message=$(printf "${error_message}${TTL_STATUS[2]}" "${upper_message}.")

    fi
    if [ -n "$not_equal_ttl_type" ]; then
        error_message="${error_message}存在es数据量多于hbase的数据类型：${not_equal_ttl_type}."
    fi
    error_message="${error_message%.}"
    if [ -n "$error_message" ]; then
        print_error_message "${TTL_STATUS[0]}" " $error_message " "${TTL_STATUS[3]}" >> $OUTPUT_FILE_ERROR_INFO_TEMP
        print_error_split >> $OUTPUT_FILE_ERROR_INFO_TEMP
    fi
}

check_es_div_core_status(){
    # 判断es,pg服务是否正常
    if [[ "$(check_server_online_all Elasticsearch)" == "Stopped" ]]; then
        return
    fi
    if [[ "$(check_pg_server_all_status)" == "Stopped" ]]; then
        return
    fi
    # 获取需要遍历的索引
    local index_each=$(cat "$APPLICATION_PROPERTIES_PATH" | grep "divide.core.names")
    index_each=$(echo "$index_each" | awk -F '=' '{print $2}' | sed 's/,/\n/g')
    local sql
    if [ $DBEVERSION -lt 3339 ]; then
        sql="select  core_type , core_name ,to_char(to_timestamp(core_time),'YYYY-MM-DD HH24:MI:SS') from tbl_es_div_core;"
    else
        sql="select  core_type , core_name ,to_char(to_timestamp(core_time / 1000),'YYYY-MM-DD HH24:MI:SS') from tbl_es_div_core;"
    fi
    local es_div_core=$(exec_sql "$sql")
    printf "【ES分core信息】：\n# exec_sql \"$sql\"\n${es_div_core}\n" >>${OUTPUT_FILE_ES}
    es_div_core=$(echo "$es_div_core" | grep -v "core_type.*core_name" | grep -v "(.*row.*)" | sed '/^$/d')
    local error_message=""
    local lack_index=""
    local line
    while IFS= read -r line; do
        local index_in_es=$(curl -XGET "http://${SLAVE_IP}:9200/_cat/indices/${line}*?v&s=creation.date.string:asc" 2>/dev/null)
        printf "【查询${line}开头的索引，按照创建时间升序排列】：\n# curl -XGET \"http://${SLAVE_IP}:9200/_cat/indices/${line}*?v&s=creation.date.string:asc\"\n${index_in_es}\n" >> $OUTPUT_FILE_ES
        if ! grep -q "sync" <<< "$line"; then
            index_in_es=$(echo "$index_in_es" | grep -v "sync")
        fi
        local last_index_in_es=$(echo "$index_in_es" | sed '1d' | tail -n 1 | awk '{print $3}')
        index_in_es=$(echo "$index_in_es" | sed '1d' | sed '$d' | awk '{print $3}' | sort)
        if [ -z "$last_index_in_es" ]; then
            lack_index="${lack_index}${line},"
            continue
        fi

        local index_in_pg=$(echo "$es_div_core" | grep " $line " | awk -F '|' '{print $2}' | sed 's/ //g' | sort)
        if [[ "$index_in_es" != "$index_in_pg" ]]; then
            local index_in_es_str=$(echo "$index_in_es" | paste -sd,)
            if [ -z "$index_in_es_str" ]; then
                index_in_es_str="$last_index_in_es"
            else
                index_in_es_str="${index_in_es_str},${last_index_in_es}"
            fi
            local index_in_pg_str=$(echo "$index_in_pg" | paste -sd,)
            if [ -z "$index_in_pg_str" ]; then
                index_in_pg_str="$last_index_in_es"
            else
                index_in_pg_str="${index_in_pg_str},${last_index_in_es}"
            fi
            local message=$(printf "${ES_DIV_CORE_STATUS[2]}" "$line" "${index_in_pg_str}" "${index_in_es_str}")
            error_message="${error_message}${message};"
        fi
    done <<< "$index_each"
    if [ -z "$error_message" ]; then
        print_normal_message "${ES_DIV_CORE_STATUS[0]}" "${ES_DIV_CORE_STATUS[1]}"
        print_normal_split
    else
        error_message="${error_message%;}"
        print_error_message "${ES_DIV_CORE_STATUS[0]}" " $error_message " "${ES_DIV_CORE_STATUS[3]}" >> $OUTPUT_FILE_ERROR_INFO_TEMP
        print_error_split >> $OUTPUT_FILE_ERROR_INFO_TEMP
    fi
    if [ -z "$lack_index" ]; then
        print_normal_message "${ES_LACK_INDEX_STATUS[0]}" "${ES_LACK_INDEX_STATUS[1]}"
        print_normal_split
    else
        lack_index="${lack_index%,}"
        local error_message=$(printf "${ES_LACK_INDEX_STATUS[2]}" "$lack_index")
        print_error_message "${ES_LACK_INDEX_STATUS[0]}" "$error_message" "${ES_LACK_INDEX_STATUS[3]}" >> $OUTPUT_FILE_ERROR_INFO_TEMP
        print_error_split >> $OUTPUT_FILE_ERROR_INFO_TEMP
    fi
}

check_patch_status(){
    # 检查环境是否有大数据补丁
    local current_patch=$(/usr/local/bin/displaypatch.sh | grep "Exterior version" | awk -F ':' '{print $2}' | sed 's/ //g')
    if [ -z "$current_patch" ]; then
        print_error_message "${PATCH_STATUS[0]}" "${PATCH_STATUS[2]}" "${PATCH_STATUS[3]}" >> $OUTPUT_FILE_ERROR_INFO_TEMP
        print_error_split >> $OUTPUT_FILE_ERROR_INFO_TEMP
    else
        local dev_version=$(/bigdata/salut/bin/common/dbcfgtool.sh -v | grep "Interior version" | awk -F ':' '{print $2}' | sed 's/ //g' | cut -c 19- | sed 's/D1/B33/g' | sed 's/S//g')
        local patch_record_file="/var/log/dblog/patch/patchrecord.log"
        local patch="$dev_version"
        local line
        if [ -f "$patch_record_file" ]; then
            local patch_records=$(cat $patch_record_file | grep "PATCHVERO=" | awk -F '=' '{print $2}' | grep "$dev_version" | uniq)
            while IFS= read -r line; do
                patch="${patch} -> ${line}"
            done <<< "$patch_records"
        fi
        local normal_message=$(printf "${PATCH_STATUS[1]}" "$current_patch")
        print_normal_message "${PATCH_STATUS[0]}" "${normal_message}"
        print_normal_split

        normal_message=$(printf "${PATCH_RECORD_STATUS[1]}" "$patch")
        print_normal_message "${PATCH_RECORD_STATUS[0]}" "${normal_message}"
        print_normal_split
    fi
}

check_ssh_copy_id_status(){
    local file_info=$(salut_ssh "ls -l /usr/bin/ssh-copy-id" sync)
    print_log "【查询ssh-copy-id文件】" >> ${OUTPUT_UPDATE_ACCESS_FILE} 2>&1
    echo "$file_info" >> ${OUTPUT_UPDATE_ACCESS_FILE} 2>&1
    print_separate_rows >> ${OUTPUT_UPDATE_ACCESS_FILE} 2>&1
    local lack_file_node=""
    local line
    local ip
    local temp_ip
    local no_power_node=""
    while IFS= read -r line; do
        temp_ip=$(echo "$line" | grep -E -o "([0-9]{1,3}\.){3}[0-9]{1,3}")
        if [ -n $temp_ip ]; then
            ip=$temp_ip
            continue
        fi
        if grep -q "No such file or directory" <<< "$line"; then
            lack_file_node="${lack_file_node}${ip},"
        elif grep -q "x" <<< "$(echo "$line" | awk '{print $1}')"; then
            no_power_node="${no_power_node}${ip},"
        fi
    done <<< "$file_info"
    lack_file_node="${lack_file_node%,}"
    no_power_node="${no_power_node%,}"
    local error_message=""
    if [ -n "$lack_file_node" ]; then
        error_message="${error_message}缺少ssh-copy-id文件的节点：${lack_file_node},"
    fi
    if [ -n "$no_power_node" ]; then
        error_message="${error_message}ssh-copy-id文件缺少执行权限的节点：${no_power_node},"
    fi
    error_message="${error_message%,}"
    if [ -z "$error_message" ]; then
        print_normal_message "${SSH_COPY_ID_STATUS[0]}" "${SSH_COPY_ID_STATUS[1]}"
        print_normal_split
    else
        print_error_message "${SSH_COPY_ID_STATUS[0]}" " ${error_message} " "${SSH_COPY_ID_STATUS[3]}" >> $OUTPUT_FILE_ERROR_INFO_TEMP
        print_error_split >> $OUTPUT_FILE_ERROR_INFO_TEMP
    fi
}

check_backup_msg_status(){
    # 单机环境校验是否有挂载冷备盘
    local normal_message=""
    local error_message=""
    local deal_idea=""
    if [ ${SLAVE_NUM} -eq 1 ]; then
        local is_exists=$(cat /etc/fstab | grep "/bigdata/backupMsg")
        if [ -z "$is_exists" ]; then
            # 未挂载冷备盘，根据现场配置规格大于0.7建议挂载冷备盘
            local total_ratio=$(exec_sql "SELECT SUM(inventory_specification_ratio) FROM tbl_data_specification" 2>&1)
            if grep -q "does not exist" <<< "$total_ratio"; then
                # 低版本不提示挂载冷备盘
                normal_message="当前环境未挂载冷备盘"
            else
                total_ratio=$(echo "$total_ratio" | sed '1,2d' | grep -vE "^$|row" | sed 's/ //g')
                if (( total_ratio > 0.7 )); then
                    error_message="当前环境未挂载冷备盘，建议增加冷备盘"
                    deal_idea="建议处理"
                fi
            fi
        else
            # 已挂载冷备盘，查看冷备盘是否识别
            local is_read=$(lsblk | grep "/bigdata/backupMsg")
            if [ -z "$is_read" ]; then
                error_message="检测到服务器存在冷备盘，但未被系统识别"
                deal_idea="必须处理"
            else
                normal_message="冷备盘挂载正常"
            fi
        fi
    else
        # 集群校验如果存在冷备盘，是否正常挂载
        local disk_info=$(salut_ssh 'cat /etc/fstab | grep "/bigdata/backupMsg";lsblk | grep "/bigdata/backupMsg"' sync)
        local error_ip=""
        local line
        local temp_ip
        local has_backup_disk="false"
        local has_backup_ip=""
        while IFS= read -r line; do
            temp_ip=$(echo "$line" | grep -E -o "([0-9]{1,3}\.){3}[0-9]{1,3}")
            if [ -n $temp_ip ]; then
                if [[ "$has_backup_disk" == "true" ]]; then
                    error_ip="${error_ip}${ip},"
                fi
                ip=$temp_ip
                has_backup_disk="false"
                continue
            fi
            if grep -q "UUID" <<< "$line"; then
                has_backup_disk="true"
                has_backup_ip="${has_backup_ip}${ip},"
            else
                has_backup_disk="false"
            fi
        done <<< "$disk_info"
        if [[ "$has_backup_disk" == "true" ]]; then
            error_ip="${error_ip}${ip},"
        fi
        error_ip="${error_ip%,}"
        if [ -n "$error_ip" ]; then
            error_message="检测到以下服务器冷备盘挂载存在问题：${error_ip}"
            deal_idea="必须处理"
        fi
        has_backup_ip="${has_backup_ip%,}"
        if [ -n "${has_backup_ip}" ]; then
            print_log "【存在冷备盘的ip：${has_backup_ip}】" >> ${OUTPUT_UPDATE_ACCESS_FILE}
        fi
    fi

    if [ -n "$normal_message" ]; then
        print_normal_message "${BACKUP_MSG_DISK_STATUS[0]}" "${normal_message}"
        print_normal_split
    fi

    if [ -n "$error_message" ]; then
        print_error_message "${BACKUP_MSG_DISK_STATUS[0]}" " ${error_message} " " ${deal_idea} " >> $OUTPUT_FILE_ERROR_INFO_TEMP
        print_error_split >> $OUTPUT_FILE_ERROR_INFO_TEMP
    fi

}

check_device_serial_number_status(){
    local normal_result=""
    local command_result=$(salut_ssh "/usr/local/hstor/driver/ENVmonitor/ENVmonitor -g -manuinfo" sync) 2>&1
    local line
    local ip
    local temp_ip
    local error_sn="true"
    local not_support_ip=""
    local error_get_ip=""
    while IFS= read -r line; do
        temp_ip=$(echo "$line" | grep -E -o "([0-9]{1,3}\.){3}[0-9]{1,3}")
        if [ -n "$temp_ip" ]; then
            if [[ "$ip" != "" ]] && [[ "$error_sn" == "true" ]]; then
                error_get_ip="${error_get_ip}${ip},"
            fi
            ip=${temp_ip}
            error_sn="true"
            continue
        fi
        if grep -q "command not found" <<< "$line"; then
            error_sn="false"
            not_support_ip="${not_support_ip}${ip},"
            continue
        elif grep -q "^DEVICE_SERIAL_NUMBER" <<< "$line"; then
            error_sn="false"
            local sn=$(echo "$line" | awk -F ':' '{print $2}')
            normal_result="${normal_result}${ip} -> ${sn},"
            continue
        fi
    done <<< "$command_result"
    if [ -n "$ip" ] & [[ "$error_sn" == "true" ]]; then
        error_get_ip="${error_get_ip}${ip},"
    fi
    normal_result="${normal_result%,}"
    not_support_ip="${not_support_ip%,}"
    error_get_ip="${error_get_ip%,}"
    if [ -n "$not_support_ip" ]; then
        print_error_message "${DEVICE_SERIAL_NUMBER_STATUS[0]}" " 以下非自研服务器无法获取sn码：${not_support_ip} " " 无需处理 " >> $OUTPUT_FILE_ERROR_INFO_TEMP
        print_error_split >> $OUTPUT_FILE_ERROR_INFO_TEMP
    fi
    if [ -n "$error_get_ip" ]; then
        print_error_message "${DEVICE_SERIAL_NUMBER_STATUS[0]}" " 以下节点获取sn码失败：${error_get_ip} " " 需要关注 " >> $OUTPUT_FILE_ERROR_INFO_TEMP
        print_error_split >> $OUTPUT_FILE_ERROR_INFO_TEMP
    fi
    if [ -n "$normal_result" ]; then
        local normal_message=$(printf "${DEVICE_SERIAL_NUMBER_STATUS[1]}" "$normal_result")
        print_normal_message "${DEVICE_SERIAL_NUMBER_STATUS[0]}" "${normal_message}"
        print_normal_split
    fi
}
check_update_before_version_status(){
    if [ $DBEVERSION -lt 3339 ]; then
        print_error_message "${UPDATE_BEFORE_VERSION_STATUS[0]}" "${UPDATE_BEFORE_VERSION_STATUS[2]}" "${UPDATE_BEFORE_VERSION_STATUS[3]}" >> $OUTPUT_FILE_ERROR_INFO_TEMP
        print_error_split >> $OUTPUT_FILE_ERROR_INFO_TEMP
        print_log "【${UPDATE_BEFORE_VERSION_STATUS[2]}】" >> ${OUTPUT_UPDATE_ACCESS_FILE}
    fi
}


open(){
    echo "System time is：$(date +'%Y-%m-%d %H:%M:%S')"
    echo
    echo -e "\033[0;36mStarting collect information ...\033[0m"
    echo
}

finish(){
    echo "【注意】脚本运行结束!"
    echo
    echo -e "\033[0;36m... Finished collect information\033[0m"
    echo
}

check_product_ver(){
    local path="/etc/product_ver"
    print_log "【检查系统是否为超融合，查询命令：cat ${path}】"

    if [ -f "$path" ];then
        local temp=$(cat $path | grep ^"CX35XXV2@")
        if [ -z "$temp" ];then
            print_log "检测结论：当前系统不是超融合系统"
        fi
    else
        print_log "检测结论：/etc目录下product_ver不存在，该系统不是超融合系统"
    fi
}

query_server_status() {
    if [[ "$is_lower_version" == "true" ]]; then
        dbserver.sh status
    else
        dbserver.sh status_detail
    fi
}

query_machine_types(){
    print_log "【DBE/F数量】"
    local machines=""
    local configs=""
    if [[ "$is_lower_version" == "true" ]]; then
        local ip
        local machine_type
        local config
        local dev_code
        while IFS= read -r ip; do
            machine_type=$(ssh -n $ip "cat ${CFGCHG_GLOBAL} | grep devVersion" | awk -F '=' '{print $2}' | awk -F '-' '{print $1}')
            dev_code=$(ssh -n $ip "cat ${CFGCHG_GLOBAL} | grep devCode" | awk -F '=' '{print $2}')
            if [ -z "$machines" ]; then
                machines="$machine_type: $dev_code   $ip"
            else
                machines="$machines\n$machine_type: $dev_code   $ip"
            fi
            config=$(ssh -n $ip "dbcfgtool.sh -q")
            config="$(print_log "【查询${ip}配置：dbcfgtool.sh -q】")\n$config\n\n\n"
            if [ -z "$configs" ]; then
                configs="$config"
            else
                configs="${configs}\n${config}"
            fi
        done <<< "$SLAVES"
    else
        machines=$(dbserver.sh status_detail 2>/dev/null | grep -E "master|slave" | awk '{print $1,$2,$3}')
        configs="$(print_log "【查询组网配置：dbcfgtool.sh -q】")\n$(salut_ssh "dbcfgtool.sh -q" sync)"
    fi
    echo -e "$machines"
    echo "================================"
    local gather=""
    echo -e "$machines" | awk '{print $1}' | sed 's/://g' | sort | uniq -c | awk '{print $2 "=" $1}'
    echo "total=$(echo -e "$machines" | wc -l)"
    print_separate_rows

    echo -e "$configs"
}

query_disk_info(){
    print_log '【磁盘挂载：salut-ssh "lsblk" sync】'
    salut_ssh "lsblk" sync
    print_separate_rows

    print_log '【磁盘挂载：salut-ssh "cat /etc/fstab" sync】'
    salut_ssh "cat /etc/fstab" sync
    print_separate_rows

    print_log "【服务器占用空间：salut-ssh \"df -h\" sync】"
    salut_ssh "df -h" sync
    print_separate_rows

    print_log "【组件数据占用磁盘空间：du -sh /bigdata/salut/data/*,du -sh /data2/* ,du -sh /index, du -sh /bigdata/*】"
    salut_ssh 'find /bigdata/salut/data -mindepth 1 -maxdepth 1 -print0 | xargs -0 -r du -sh' sync 2>&1
    print_separate_rows

    salut_ssh 'find /data2 -mindepth 1 -maxdepth 1 -print0 | xargs -0 -r du -sh' sync 2>&1
    print_separate_rows

    salut_ssh 'du -sh /index' sync 2>&1
    print_separate_rows

    salut_ssh 'find /bigdata -mindepth 1 -maxdepth 1 -print0 | xargs -0 -r du -sh' sync 2>&1
    print_separate_rows
}

query_hdfs_size(){
    print_log "【HDFS占用空间】"
    print_log "【hdfs dfs -du -h /】"
    hdfs dfs -du -h / 2>&1
    print_separate_rows

    print_log "【hdfs dfs -du -h /hbase/data/default】"
    hdfs dfs -du -h /hbase/data/default 2>&1
    print_separate_rows

    print_log "【hdfs dfs -du -h /salut】"
    hdfs dfs -du -h /salut 2>&1
    print_separate_rows

    print_log "【hdfs dfs -du -h /salut/traffic】"
    hdfs dfs -du -h /salut/traffic 2>&1
    print_separate_rows

    print_log "【hdfs dfs -du -h /salut/stk】"
    hdfs dfs -du -h /salut/stk 2>&1
    print_separate_rows
}

# 查询主机版本补丁信息
version(){
    print_log "【主版本号以及产品信息，查询命令：dbcfgtool.sh -v】"
    dbcfgtool.sh -v
    print_separate_rows

    print_log "【补丁信息，查询命令：displaypatch.sh】"
    displaypatch.sh
    print_separate_rows

    print_log "【服务状态，查询命令：dbserver.sh status_detail/dbserver.sh status】"
    query_server_status
    print_separate_rows
}

ssh_version(){
    print_log "【ssh版本：ssh -V】"
    salut_ssh "ssh -V" sync
    print_separate_rows
}

# 硬件信息
basic_info(){
    # todo 系统混用的集群无法收集完全
    echo "=================================================== 【系统版本，查询命令：salut-ssh 'cat /etc/redhat-release' sync】"
    # 查询 CentOS 系统信息
    if [ -f /etc/redhat-release ]; then
        salut_ssh 'cat /etc/redhat-release' sync  2>/dev/null
    # 查询欧拉系统信息
    elif [ -f /etc/os-release ]; then
        salut_ssh 'cat /etc/os-release' sync  2>/dev/null
    # 查询麒麟系统信息
    elif command -v nkvers >/dev/null 2>&1; then
        salut_ssh 'nkvers' sync  2>/dev/null
    else
        echo "无法确定操作系统类型"
    fi
    echo
    echo "=================================================== 【系统运行时间，查询命令：salut-ssh 'uptime' sync】"
    salut_ssh 'uptime' sync 2>/dev/null
    echo
    echo "=================================================== 【本地时间，查询命令：salut-ssh 'date' sync】"
	salut_ssh 'date' sync 2>/dev/null
    echo
    echo "=================================================== 【分区信息，查询命令：salut-ssh 'lsblk' sync】"
    salut_ssh 'lsblk' sync
    echo
    echo "=================================================== 【内存详情，查询命令：salut-ssh 'free -g' sync】"
	salut_ssh 'free -g' sync
    echo
    ssh_version
    print_log '【SN码获取：salut-ssh "/usr/local/hstor/driver/ENVmonitor/ENVmonitor -g -manuinfo" sync'
    salut_ssh "/usr/local/hstor/driver/ENVmonitor/ENVmonitor -g -manuinfo" sync
    print_separate_rows
}

# ES中cluster_uuid
es_uuid(){
    printf "【ES uuid信息】：\n# curl -X GET -s http://node:9200\n"
    for node in ${ALL_NODES}; do
        curl -X GET -s http://${node}:9200 2>/dev/null
    done
}

# GP 各 各种表 详情和总数
gp_table_status(){
    local partition_table_detail=$(/bigdata/salut/components/postgres/pgsql/bin/psql -U gpadmin -d warehouse -p 6543 -h node1 --command "select partitiontablename,partitionname,partitionlevel,tablename from pg_partitions order by tablename;" 2>/dev/null)
    local partition_table_info=$(/bigdata/salut/components/postgres/pgsql/bin/psql -U gpadmin -d warehouse -p 6543 -h node1 --command "select tablename,count(*) from pg_partitions group by tablename;" 2>/dev/null)
    echo -e "【GP 各种表的分表详情】：\n# select partitiontablename,partitionname,partitionlevel,tablename from pg_partitions order by tablename\n${partition_table_detail}"
    echo -e "【GP 各种表的分表汇总】：\n# select tablename,count(*) from pg_partitions group by tablename\n${partition_table_info}"
    echo
}

# 查询表tbl_data_specification
ttl_info() {
    print_log "【PG数据库 tbl_data_specification 表】："
    exec_sql "select * from tbl_data_specification;"
    print_separate_rows

    print_log "【PG数据库 tbl_ttl_config 表】："
    exec_sql "select * from tbl_ttl_config;"
    print_separate_rows
}

# Yarn 的状态
block_task(){
  echo
  echo "==================================== 【Yarn 任务状态】："
  echo
  curl http://${MASTER_IP}:8188/ws/v1/cluster/appstatistics 2>/dev/null
  echo
}

yarn_task_details(){
  echo
  echo "==================================== 【Yarn 任务详情】："
  echo
  curl http://${MASTER_IP}:8188/ws/v1/cluster/apps 2>/dev/null
  echo
}

# yarn端口在55版本已经修改
yarn_port_info(){
  echo
  echo "==================================== 【Yarn 端口信息】："
  echo
  cat /bigdata/salut/components/hadoop/etc/hadoop/yarn-site.xml | awk '/:[0-9]+/'| cut -d ':' -f 2 | cut -d '<' -f 1 2>/dev/null
  echo
}

# 收集的规格配置信息
specification_info() {
  echo
  echo "==================================== 【规格配置信息】："
  echo
	exec_sql 'select tds.description as "数据类型", tds.max_specification as "秒级检索规格（万）", tds.curr_specification as "当前秒级天梭规格（万）", tds.specification_ratio as "秒级检索规格占比", tds.avg_inventory as "每天数量（万）", ttc.ttl_value as "留存期（/天）", tds.max_inventory_specification as "库容规格（万）", tds.curr_inventory_specification as "当前库容规格（万）", tds.inventory_specification_ratio as "库容规格占比" from tbl_data_specification tds left join tbl_ttl_config ttc on  ttc.ttl_data_type = tds.data_type;'
  echo
}

# 收集全部组件信息（仅组件信息，不收集日志）
collect_component(){
    es_uuid >>${OUTPUT_FILE_ES}
    gp_table_status >>${OUTPUT_FILE_GP}
    block_task >>${OUTPUT_FILE_YARN}
    yarn_task_details >>${OUTPUT_FILE_YARN}
    yarn_port_info >>${OUTPUT_FILE_YARN}
    ttl_info >>${OUTPUT_FILE_TTL}
    specification_info >>${OUTPUT_FILE_TTL}
}

collect_config(){
    local ip=$1
    local destination_path=$2
    # 定时调度任务
    scp root@${ip}:/etc/crontab "$destination_path" 2>/dev/null
    # db配置文件
    scp root@${ip}:/bigdata/salut/conf/db*.conf "$destination_path" 2>/dev/null
    scp root@${ip}:/bigdata/salut/components/jetty/resources/application.properties "$destination_path" 2>/dev/null
    scp root@${ip}:$FLUME_CONF_PROPERTIES "$destination_path" 2>/dev/null
    scp root@${ip}:/bigdata/salut/conf/kafka/kafkaCreate.sh "$destination_path" 2>/dev/null
}

collect_install_log(){
    local ip=$1
    local destination_path=$2
    # 直装
    scp root@${ip}:/var/log/dblog/dbinstall.log* "${destination_path}" 2>/dev/null
    # 升级
    scp root@${ip}:/var/log/dblog/dbupdate.log* "${destination_path}" 2>/dev/null
    # 打补丁
    scp root@${ip}:/var/log/dblog/patch/dbpatchinstall.log* "${destination_path}" 2>/dev/null
    # 补丁安装记录
    scp root@${ip}:/var/log/dblog/patch/patchrecord.log* "${destination_path}" 2>/dev/null
    # 服务起停
    scp root@${ip}:/var/log/dblog/dbserver/* "${destination_path}" 2>/dev/null
    # cron日志
    scp root@${ip}:/var/log/cron* "${destination_path}" 2>/dev/null
}

collect_service_log(){
    local ip=$1
    local destination_path=$2
    # rest接口
    scp root@${ip}:/var/log/dblog/salut/web/jetty.log* "${destination_path}" 2>/dev/null
    scp root@${ip}:/var/log/dblog/salut/web/salut-rest.log* "${destination_path}" 2>/dev/null
    # thrift接口
    scp root@${ip}:/var/log/dblog/salut/thrift/salut-thrift-slave.log* "${destination_path}" 2>/dev/null
    # service接口
    scp root@${ip}:/var/log/dblog/salut/service/salut-service.log* "${destination_path}" 2>/dev/null
    # flume接口
    scp root@${ip}:/var/log/dblog/salut/flume/flume.log* "${destination_path}" 2>/dev/null
    # Tool日志
    scp root@${ip}:/var/log/dblog/salut/tools/salut-* "${destination_path}" 2>/dev/null
}

collect_system_sa_log(){
    local ip=$1
    local destination_path=$2
    ssh -n ${ip} "ls -l --time-style='+%Y-%m-%d %H:%M' /var/log/sa >/var/log/sa/sa_time.txt" 2>/dev/null
    scp -r root@${ip}:/var/log/sa/ "${destination_path}" 2>/dev/null
    ssh -n ${ip} "rm -f /var/log/sa/sa_time.txt" 2>/dev/null
}

collect_message_log(){
    local ip=$1
    local destination_path=$2
    scp -r root@${ip}:/var/log/message* "${destination_path}" 2>/dev/null
}

tar_dir_greater_100m(){
    local dir="$1"
    local size_100m=102400
    local size=$(du -s "$dir" | awk '{print $1}')
    if (( size > size_100m )); then
        cd "$dir" && tar -zcvf collect.tar.gz * 1>/dev/null
        if [ $? -eq 0 ]; then
            mkdir -p "${dir}_temp"
            mv "${dir}/collect.tar.gz" "${dir}_temp"
            rm -rf "${dir}"
            mv "${dir}_temp" "${dir}"
        fi
    fi
}

collect_info(){
    version >>${OUTPUT_SOFTWARE_INFO} 2>&1
    check_product_ver >>${OUTPUT_SOFTWARE_INFO} 2>&1
    basic_info >>${OUTPUT_HARDWARE_INFO} 2>/dev/null
    # 03-全部组件信息
    collect_component 2>/dev/null
    # 01-日志与配置信息
    local line
    while IFS= read -r line; do
        local ip=$(echo "$line" | awk -F = '{print $2}')
        local config_path="${OUTPUT_PATH_CONFIG}/${line}"
        collect_config "$ip" "$config_path"
        local server_log_path="${config_path}/${SERVER_LOG_DIR}"
        collect_install_log "$ip" "$server_log_path"
        local service_log_path="${config_path}/${SERVICE_LOG_DIR}"
        collect_service_log "$ip" "$service_log_path"
        local system_log_path="${config_path}/${SYSTEM_LOG_DIR}"
        collect_system_sa_log "$ip" "$system_log_path"
        collect_message_log "$ip" "$system_log_path"
        tar_dir_greater_100m "$config_path"
    done <<< "$ALL_HOST_IP"
    # 02-节点详细信息
    while IFS= read -r line; do
        local ip=$(echo "$line" | awk -F = '{print $2}')
        local node_path="${OUTPUT_NODE_PATH}/${line}"
        # 调用每个节点中cluster-info-collector-v##.sh脚本中node_check方法，收集当前节点信息
        timeout 600s ssh -n $ip "sh ${COLLECTOR_TOOL_PATH} -node ${node_path} >/dev/null"
        tar_dir_greater_100m "$node_path"
        if [[ "$CURRENT_IP" != "$ip" ]]; then
            ssh -n $ip "rm -rf ${COLLECTOR_TOOL_PATH}"
        fi
    done <<< "$ALL_HOST_IP" &
    wait
}

collect_current_es_size(){
    local index_in_es=$(curl -XGET "http://${SLAVE_IP}:9200/_cat/indices?h=index" 2>/dev/null)
    index_in_es=$(echo "$index_in_es" | grep -v "\.")
    local result=""
    local index
    while IFS= read -r index; do
        local count=$(curl -XGET "http://${SLAVE_IP}:9200/$index/_count" 2>&1)
        count=$(echo "$count" | sed 's/,/\n/g' | grep "\"count\"" | awk -F ':' '{print $2}')
        if [ -z "$result" ]; then
            result="${index} ${count}"
        else
            result="${result}\n${index} ${count}"
        fi
    done <<< "$index_in_es"
    print_log "【查询es数据量：curl -XGET \"http://${SLAVE_IP}:9200/_cat/indices\"】"
    curl -XGET "http://${SLAVE_IP}:9200/_cat/indices" 2>&1
    echo "======================================================================================================"
    echo -e "$result" | awk '
    {
        # 删除第一列末尾的数字
        key = $1
        gsub(/[0-9]+$/, "", key)  # 去掉第一列末尾的数字

        # 累加第二列的值
        sum[key] += $2
    }
    END {
        # 输出结果
        for (k in sum) {
            print k "=" sum[k]
        }
    }' | sort
    print_separate_rows

    print_log "【查询pg tbl_es_div_core表】"
    exec_sql "SELECT * FROM tbl_es_div_core" 2>&1
    print_separate_rows
}


collect_update_assess_info(){
    # 版本、补丁、服务状态
    version >> ${OUTPUT_UPDATE_ACCESS_FILE} 2>&1

    # DBE/DBF
    query_machine_types >> ${OUTPUT_UPDATE_ACCESS_FILE} 2>&1

    # 磁盘挂载、服务器占用空间
    query_disk_info >> ${OUTPUT_UPDATE_ACCESS_FILE} 2>&1

    # hdfs各目录占用空间
    query_hdfs_size >> ${OUTPUT_UPDATE_ACCESS_FILE} 2>&1

    # 规格配置
    local ttl_data_type_exist=$(exec_sql "select 1 from information_schema.columns where table_name='tbl_ttl_config' and column_name='ttl_data_type'")
    if grep -q "(0 rows)" <<< "$ttl_data_type_exist"; then
        ttl_info >> ${OUTPUT_UPDATE_ACCESS_FILE} 2>&1
    else
        specification_info >> ${OUTPUT_UPDATE_ACCESS_FILE} 2>&1
    fi

    # es数据量
    collect_current_es_size >> ${OUTPUT_UPDATE_ACCESS_FILE} 2>&1

    # ssh版本
    ssh_version >> ${OUTPUT_UPDATE_ACCESS_FILE} 2>&1
}

####################################################################################################################################################################
####################################################################################################################################################################
# 巡检主函数
####################################################################################################################################################################
####################################################################################################################################################################
# 主函数：只有主机执行main函数中的收集异常信息函数，第一层级信息收集
main(){
    # 删除遗留文件夹
    rm -rf ${OUTPUT_PATH_MASTER} 2>/dev/null
    # 提前创建文件夹
    mkdir -p ${OUTPUT_PATH_CONFIG}
    mkdir -p ${OUTPUT_NODE_PATH}
    mkdir -p ${OUTPUT_PATH_COMPONENTS}
    mkdir -p ${OUTPUT_PATH_UPDATE_ACCESS}
    local line
    while IFS= read -r line; do
        mkdir -p "${OUTPUT_PATH_CONFIG}/${line}/${SERVER_LOG_DIR}"
        mkdir -p "${OUTPUT_PATH_CONFIG}/${line}/${SERVICE_LOG_DIR}"
        mkdir -p "${OUTPUT_PATH_CONFIG}/${line}/${SYSTEM_LOG_DIR}"
        mkdir -p "${OUTPUT_NODE_PATH}/${line}"
    done <<< "$ALL_HOST_IP"

    salut_ssh "mkdir -p ${CURRENT_DIR}" >/dev/null 2>&1

    salut_cp $COLLECTOR_TOOL_PATH >/dev/null 2>&1

    # 巡检环境
    merge_success_items | tee -a $OUTPUT_FILE_SUCCESS_INFO_TEMP
    merge_error_message | tee -a $OUTPUT_FILE_CLUSTER_INFO
    cat $OUTPUT_FILE_SUCCESS_INFO_TEMP >> $OUTPUT_FILE_CLUSTER_INFO
    local temp_result=$(cat "$OUTPUT_FILE_CLUSTER_INFO" | sed 's/^.*\x1b\[1K\r//' | sed 's/正在检查[^C]*C//g' | sed -r "s/\x1B\[[0-9;]*[mGKF]//g")
    echo "$temp_result" > $OUTPUT_FILE_CLUSTER_INFO
    rm -f $OUTPUT_FILE_SUCCESS_INFO_TEMP

    if [ -f $OUTPUT_STATISTICS_FILE ]; then
        cat $OUTPUT_STATISTICS_FILE
    fi

    # 收集信息
    echo -n "正在收集信息中..."
    collect_update_assess_info >/dev/null
    collect_info >/dev/null
    echo -ne "\033[1K\r"

    # 打包收集的文件
    tar -zcvf ${OUTPUT_TAR} -C ${OUTPUT_PATH_MASTER} . >/dev/null 2>&1

    echo "+------------------------------------------------------------------------------------------------+"
    echo "|压缩包地址：${OUTPUT_TAR}                                  |"
    echo "+------------------------------------------------------------------------------------------------+"

}









####################################################################################################################################################################
####################################################################################################################################################################
# 异常服务检测报告
####################################################################################################################################################################
####################################################################################################################################################################


# 检测主机异常服务信息
CURRENT_DIR="$(cd "$( dirname "${BASH_SOURCE[0]}" )" >/dev/null 2>&1 && pwd)"

printLine() {
  echo "---------------------------------------------------"
}

####################################################################################################################################################################
####################################################################################################################################################################
# 巡检已经开发完，未使用功能：拉起服务
####################################################################################################################################################################
####################################################################################################################################################################
# 尝试拉起对应异常的服务 -- 封装拉服务
# 处理 SSH 连接
ssh_needed(){
    local node=$1
    # 根据节点名称判断是否需要 SSH 连接
    # 在这里根据你的逻辑来判断，例如：如果节点名称是 node1，则返回 true，否则返回 false
    if [ "$node" == "node1" ]; then
        return 1  # 不需要 SSH 连接
    else
        return 0  # 需要 SSH 连接
    fi
}
# 重新启动服务函数
start_service() {
    local node="$1"
    local service="$2"

    if [ "$node" == "node1" ]; then
        # 直接拉起服务
        if dbserver.sh start_server "$service"; then
            echo "$node：$service 服务重启成功"
            echo
        else
            echo "$node：$service 服务重启失败"
            echo
        fi
    else
        # 其他节点需要通过 SSH 连接拉起服务
        if ssh $node dbserver.sh start_server "$service"; then
            echo "$node：$service 服务重启成功"
            echo
        else
            echo "$node：$service 服务重启失败"
            echo
        fi
    fi
}
# 主函数，将调用其他函数
abnormal_start() {
    local services_to_start="$stopped_server"
    local started_nodes=""
    local wait_time=10  # 等待时间，单位：秒

    while IFS= read -r stopped_line; do
        if [ -z "$stopped_line" ]; then
            continue  # 跳过空行
        fi

        local node=$(echo "$stopped_line" | awk -F ':' '{print $1}')
        local services=$(echo "$stopped_line" | awk -F ':' '{print $2}')

        # 如果该节点已经被重新启动过了，则跳过
        if echo "$started_nodes" | grep -q "$node"; then
            continue
        fi

        # 检查该节点所需的服务是否已经被重新启动过了，如果是则跳过该节点
        if echo "$services_to_start" | grep -q "$node"; then
            echo "$node 需要拉起的服务有: $services"

            # 检查是否需要 SSH 连接到节点
            if ssh_needed "$node"; then
                # 禁用SSH连接输出
                exec 3>&1  # 将文件描述符 3 设置为标准输出
                exec &> /dev/null  # 将标准输出重定向到 /dev/null
                echo "node1 SSH连接 $node"
                if ssh $node; then
                    echo "连接成功"
                else
                    echo "连接失败，跳过服务重新启动"
                    continue  # 跳过本次循环，继续下一个节点
                fi
                # 重新启用SSH连接输出
                exec >&3  # 恢复标准输出
                exec 3>&-  # 关闭文件描述符 3
            fi

            # 按顺序逐个拉起服务
            IFS=',' read -ra services_array <<< "$services"
            for service in "${services_array[@]}"; do
                echo "$(date +'%Y-%m-%d %H:%M')"
                start_service "$node" "$service"
                sleep "$wait_time"  # 等待一段时间再继续下一个服务
            done

            # 如果通过 SSH 连接进去了，记得退出连接
            if ssh_needed "$node"; then
                exec 3>&1  # 将文件描述符 3 设置为标准输出
                exec &> /dev/null  # 将标准输出重定向到 /dev/null
                echo "关闭node1 SSH连接 $node"
                if ssh $node exit; then
                    echo "成功关闭 node1 SSH连接 $node"
                else
                    echo "未能关闭 node1 SSH连接 $node"
                fi
                # 重新启用SSH连接输出
                exec >&3  # 恢复标准输出
                exec 3>&-  # 关闭文件描述符 3
            fi
        fi

    done <<< "$stopped_server"
}

# 尝试拉起对应异常服务 -- 底层指令拉起 --准备开发
# 尝试拉起对应异常服务 -- 底层指令拉起 --准备开发
# 尝试拉起对应异常服务 -- 底层指令拉起 --准备开发











####################################################################################################################################################################
####################################################################################################################################################################
# 基础环境信息报告
####################################################################################################################################################################
####################################################################################################################################################################



####################################################################################################################################################################
####################################################################################################################################################################
# 全部组件信息-参数
####################################################################################################################################################################
####################################################################################################################################################################
collect_component_all(){
  pid=$$
  another_pid=$(ps -ef | grep -v $pid | grep $0 | grep -v grep | awk '{print $2}')
  if [[ $another_pid ]];then
    echo "检测到此脚本正在运行,如需重新运行请kill掉此前的进程"
    exit 1
  fi
  # 删除遗留文件夹
  rm -rf ${OUTPUT_PATH_MASTER} 2>/dev/null
  # 提前创建文件夹
  mkdir -p ${OUTPUT_PATH_MASTER}/temp

  # 信息收集开始
  open
  echo "【注意】仅收集全部组件信息，不影响业务的正常使用！"

  # 收集全部组件信息 
  collect_component 2>/dev/null
  wait

  rm -rf ${OUTPUT_PATH_MASTER}/temp

  # 总打包
  tar zcvf /home/Collect-ComponentInfo-`date +%Y-%m-%d-%H-%M-%S`.tgz -C ${OUTPUT_PATH_MASTER} . 1>/dev/null
  wait

  finish
  echo "############################# COLLECTION INFO ##################################"
  echo -e "取包地址：[cd /home]，压缩包名称：\033[0;31mCollect-ComponentInfo-yyyy-mm-dd-h-m-s.tgz\033[0m"
  echo "################################################################################"
  wait
  
  #echo 正在删除遗留文件
  rm -rf ${OUTPUT_PATH_MASTER}
}









####################################################################################################################################################################
####################################################################################################################################################################
# 日志与配置信息（收集对应异常服务的日志）
####################################################################################################################################################################
####################################################################################################################################################################
# 服务和异常函数映射关系
declare -A service_log_functions=(
    #["Monitor"]="monitor_log"
    ["Zookeeper"]="zookeeper_log"
    ["Redis"]="redis_log"
    ["Jetty"]="jetty_log"
    ["Elasticsearch"]="es_log"
    ["Flume"]="flume_log"
    ["Bgclient"]="bgclient_log"
    ["Yarn-NodeManager"]="yarn_log"
    ["Yarn-ResourceManager"]="yarn_log"
    #["OperationTools"]="operation_log"
    ["HBase-RegionServer"]="hbase_log"
    ["HBase-Master"]="hbase_log"
    ["Nginx"]="nginx_log"
    ["PostgreSQL"]="pg_log"
    ["HDFS-DataNode"]="hdfs_log"
    ["HDFS-NameNode"]="hdfs_log"
    ["HDFS-SecondaryNameNode"]="hdfs_log"
    ["GreenPlum"]="gp_log"
    ["SlaveServer"]="miserver_log"
    ["IrisServer"]="miserver_log"
    ["MasterServer"]="miserver_log"
    #["Kibana"]="kibana_log"
    ["Kafka"]="kafka_log"
    #["FeatureSlave"]="feature_log"
    #["FeatureMaster"]="feature_log"
    #["SynDisposeRule"]="SynDisposeRule_log"
    #["TimeTask"]="timeTask_log"
)

####################################################################################################################################################################
####################################################################################################################################################################
# 收集全部服务的日志-参数
####################################################################################################################################################################
####################################################################################################################################################################
component_log(){

  pid=$$
  another_pid=$(ps -ef | grep -v $pid | grep $0 | grep -v grep | awk '{print $2}')
  if [[ $another_pid ]];then
    echo "检测到此脚本正在运行,如需重新运行请kill掉此前的进程"
    exit 1
  fi
  # 删除遗留文件夹
  rm -rf ${OUTPUT_PATH_MASTER} 2>/dev/null
  # 提前创建文件夹
  mkdir -p ${OUTPUT_PATH_MASTER}/01-日志与配置信息
  # 执行当前信息收集脚本的节点，一般为主节点
  CUR_NODE_IP=$(hostname -i)

  # 信息收集开始
  open
  echo "【注意】仅收集配置与组件日志文件，不影响业务的正常使用！"
  # 组件信息
  port_log > /dev/null 2>&1
  echo "$(date +'%Y-%m-%d %H:%M') 收集配置日志完成-port"

  zookeeper_log > /dev/null 2>&1
  echo "$(date +'%Y-%m-%d %H:%M') 收集组件日志完成-ZOOKEEPER"
  redis_log > /dev/null 2>&1
  echo "$(date +'%Y-%m-%d %H:%M') 收集组件日志完成-REDIS"
  jetty_log > /dev/null 2>&1
  echo "$(date +'%Y-%m-%d %H:%M') 收集组件日志完成-JETTY"
  es_log > /dev/null 2>&1
  echo "$(date +'%Y-%m-%d %H:%M') 收集组件日志完成-ES"
  flume_log > /dev/null 2>&1
  echo "$(date +'%Y-%m-%d %H:%M') 收集组件日志完成-FLUME"
  bgclient_log > /dev/null 2>&1
  echo "$(date +'%Y-%m-%d %H:%M') 收集组件日志完成-BGCLIENT"
  yarn_log > /dev/null 2>&1
  echo "$(date +'%Y-%m-%d %H:%M') 收集组件日志完成-YARN"
  hbase_log > /dev/null 2>&1
  echo "$(date +'%Y-%m-%d %H:%M') 收集组件日志完成-HBASE"
  nginx_log > /dev/null 2>&1
  echo "$(date +'%Y-%m-%d %H:%M') 收集组件日志完成-NGINX"
  pg_log > /dev/null 2>&1
  echo "$(date +'%Y-%m-%d %H:%M') 收集组件日志完成-PG"
  hdfs_log > /dev/null 2>&1
  echo "$(date +'%Y-%m-%d %H:%M') 收集组件日志完成-HDFS"
  gp_log > /dev/null 2>&1
  echo "$(date +'%Y-%m-%d %H:%M') 收集组件日志完成-GP"
  miserver_log > /dev/null 2>&1
  echo "$(date +'%Y-%m-%d %H:%M') 收集组件日志完成-MISSERVER"
  kafka_log > /dev/null 2>&1
  echo "$(date +'%Y-%m-%d %H:%M') 收集组件日志完成-KAFKA"
  wait

  # 压缩文件夹
  for dir in /${OUTPUT_PATH_MASTER}/01-日志与配置信息/*/
  do
    dir_name=$(basename "$dir")
    tar -czvf "/${OUTPUT_PATH_MASTER}/01-日志与配置信息/${dir_name}.tar.gz" -C "/${OUTPUT_PATH_MASTER}/01-日志与配置信息" "$dir_name" 1>/dev/null
    if [ $? -eq 0 ]; then
        rm -rf "${dir}"
    else
        echo "压缩 ${dir} 失败，未删除原始目录"
    fi
  done
  wait

  # 总打包
  wait
  tar zcvf /home/Collect-ComponentLog-`date +%Y-%m-%d-%H-%M-%S`.tgz -C ${OUTPUT_PATH_MASTER} . 1>/dev/null
  wait

  finish
  echo "############################# COLLECTION INFO ##################################"
  echo -e "取包地址：[cd /home]，压缩包名称：\033[0;31mCollect-ComponentLog-yyyy-mm-dd-h-m-s.tgz\033[0m"
  echo "################################################################################"
  
  #echo 正在删除遗留文件
  rm -rf ${OUTPUT_PATH_MASTER}
}









####################################################################################################################################################################
####################################################################################################################################################################
# 调用全部服务日志函数
####################################################################################################################################################################
####################################################################################################################################################################






#----------------------------------------------------------------------------------------------------------------------------------------------
# 组件类日志
# Zookeeper相关日志和配置文件
zookeeper_log(){
    for node in ${ALL_NODES}; do
    {
        node_number=$(dbcfgtool.sh -q | grep ${node} | grep -E '^node[0-9]+=')
        # 提前建立好相应文件夹
        mkdir -p ${OUTPUT_PATH_MASTER}/01-日志与配置信息/${node_number}/zookeeper        
        # 收集指定节点的 Zookeeper 日志并传输到 node1       
        scp root@${node}:/var/log/dblog/zookeeper/zookeeper-root-server-node* ${OUTPUT_PATH_MASTER}/01-日志与配置信息/${node_number}/zookeeper 2>/dev/null
        scp root@${node}:/bigdata/salut/components/zookeeper/conf/zoo.cfg* ${OUTPUT_PATH_MASTER}/01-日志与配置信息/${node_number}/zookeeper 2>/dev/null
        # 检查传输是否成功
        #if [ $? -eq 0 ]; then      
        #    echo "成功传输 ${node_number} 的 zookeeper 日志到 node1" 
        #else
        #    echo "未能传输 ${node_number} 的 zookeeper 日志到 node1"           
        #fi
    }
    done
}

#----------------------------------------------------------------------------------------------------------------------------------------------
# Redis相关日志和配置文件
redis_log(){
    for node in ${ALL_NODES}; do
    {
        node_number=$(dbcfgtool.sh -q | grep ${node} | grep -E '^node[0-9]+=')
        # 提前建立好相应文件夹
        mkdir -p ${OUTPUT_PATH_MASTER}/01-日志与配置信息/${node_number}/redis        
        # 收集指定节点的 Redis 日志并传输到 node1       
        scp -r root@${node}:/var/log/dblog/redis/ ${OUTPUT_PATH_MASTER}/01-日志与配置信息/${node_number}/redis 2>/dev/null
        scp -r root@${node}:/bigdata/salut/components/redis/redis-cluster/ ${OUTPUT_PATH_MASTER}/01-日志与配置信息/${node_number}/redis 2>/dev/null
    }
    done
}

#----------------------------------------------------------------------------------------------------------------------------------------------
# Jetty相关日志和配置文件
jetty_log(){
    for node in ${ALL_NODES}; do
    {
        node_number=$(dbcfgtool.sh -q | grep ${node} | grep -E '^node[0-9]+=')
        # 提前建立好相应文件夹
        mkdir -p ${OUTPUT_PATH_MASTER}/01-日志与配置信息/${node_number}/jetty        
        # 收集指定节点的 Jetty 日志并传输到 node1       
        scp root@${node}:/var/log/dblog/salut/web/jetty.log* ${OUTPUT_PATH_MASTER}/01-日志与配置信息/${node_number}/jetty 2>/dev/null
        scp root@${node}:/bigdata/salut/sbin/stk/startJetty.sh* ${OUTPUT_PATH_MASTER}/01-日志与配置信息/${node_number}/jetty 2>/dev/null
        
    }
    done
}

#----------------------------------------------------------------------------------------------------------------------------------------------
# ES相关日志和配置文件
es_log(){
    for node in ${ALL_NODES}; do
    {
        node_number=$(dbcfgtool.sh -q | grep ${node} | grep -E '^node[0-9]+=')
        # 提前建立好相应文件夹
        mkdir -p ${OUTPUT_PATH_MASTER}/01-日志与配置信息/${node_number}/elasticsearch        
        # 收集指定节点的 ES 日志并传输到 node1       
        scp root@${node}:/var/log/dblog/es/logs/elastic_cluster* ${OUTPUT_PATH_MASTER}/01-日志与配置信息/${node_number}/elasticsearch 2>/dev/null
        scp root@${node}:/bigdata/salut/components/es/config/elasticsearch.yml* ${OUTPUT_PATH_MASTER}/01-日志与配置信息/${node_number}/elasticsearch 2>/dev/null
        
    }
    done
}

#----------------------------------------------------------------------------------------------------------------------------------------------
# Flume相关日志和配置文件
flume_log(){
    for node in ${ALL_NODES}; do
    {
        node_number=$(dbcfgtool.sh -q | grep ${node} | grep -E '^node[0-9]+=')
        # 提前建立好相应文件夹
        mkdir -p ${OUTPUT_PATH_MASTER}/01-日志与配置信息/${node_number}/flume        
        # 收集指定节点的 Flume 日志并传输到 node1       
        scp root@${node}:/var/log/dblog/salut/flume/flume.log* ${OUTPUT_PATH_MASTER}/01-日志与配置信息/${node_number}/flume 2>/dev/null
        scp root@${node}:/bigdata/salut/components/flume/conf/flume-conf.properties* ${OUTPUT_PATH_MASTER}/01-日志与配置信息/${node_number}/flume 2>/dev/null
        scp root@${node}:/bigdata/salut/components/flume/conf/flume-env.sh* ${OUTPUT_PATH_MASTER}/01-日志与配置信息/${node_number}/flume 2>/dev/null
        # 检查传输是否成功
        
    }
    done
}

#----------------------------------------------------------------------------------------------------------------------------------------------
# Bgclient相关日志和配置文件
bgclient_log(){
    for node in ${ALL_NODES}; do
    {
        node_number=$(dbcfgtool.sh -q | grep ${node} | grep -E '^node[0-9]+=')
        # 提前建立好相应文件夹
        mkdir -p ${OUTPUT_PATH_MASTER}/01-日志与配置信息/${node_number}/bgclient        
        # 收集指定节点的 Bgclient 日志并传输到 node1       
        scp -r root@${node}:/var/log/dblog/sipclient/ ${OUTPUT_PATH_MASTER}/01-日志与配置信息/${node_number}/bgclient 2>/dev/null
        # 检查传输是否成功
        
    }
    done
}

#----------------------------------------------------------------------------------------------------------------------------------------------
# Yarn相关日志和配置文件
yarn_log(){
    for node in ${ALL_NODES}; do
    {
        node_number=$(dbcfgtool.sh -q | grep ${node} | grep -E '^node[0-9]+=')
        # 提前建立好相应文件夹
        mkdir -p ${OUTPUT_PATH_MASTER}/01-日志与配置信息/${node_number}/yarn        
        # 收集指定节点的 Yarn 日志并传输到 node1       
        scp root@${node}:/var/log/dblog/hadoop/yarn-root-nodemanager-node* ${OUTPUT_PATH_MASTER}/01-日志与配置信息/${node_number}/yarn 2>/dev/null
        scp root@${node}:/var/log/dblog/hadoop/yarn-root-resourcemanager-node* ${OUTPUT_PATH_MASTER}/01-日志与配置信息/${node_number}/yarn 2>/dev/null
        scp root@${node}:/bigdata/salut/components/hadoop/etc/hadoop/yarn-site* ${OUTPUT_PATH_MASTER}/01-日志与配置信息/${node_number}/yarn 2>/dev/null
        scp root@${node}:/bigdata/salut/components/hadoop/etc_second/hadoop/yarn-site* ${OUTPUT_PATH_MASTER}/01-日志与配置信息/${node_number}/yarn 2>/dev/null
        
    }
    done
}

#----------------------------------------------------------------------------------------------------------------------------------------------
# Hbase相关日志和配置文件
hbase_log(){
    for node in ${ALL_NODES}; do
    {
        node_number=$(dbcfgtool.sh -q | grep ${node} | grep -E '^node[0-9]+=')
        # 提前建立好相应文件夹
        mkdir -p ${OUTPUT_PATH_MASTER}/01-日志与配置信息/${node_number}/hbase        
        # 收集指定节点的 Hbase 日志并传输到 node1       
        scp root@${node}:/var/log/dblog/hbase/hbase-root-master-node* ${OUTPUT_PATH_MASTER}/01-日志与配置信息/${node_number}/hbase 2>/dev/null
        scp root@${node}:/var/log/dblog/hbase/hbase-root-regionserver-node* ${OUTPUT_PATH_MASTER}/01-日志与配置信息/${node_number}/hbase 2>/dev/null
        scp root@${node}:/bigdata/salut/components/hbase/conf/hbase-site* ${OUTPUT_PATH_MASTER}/01-日志与配置信息/${node_number}/hbase 2>/dev/null
        scp root@${node}:/bigdata/salut/components/hbase/conf/regionservers* ${OUTPUT_PATH_MASTER}/01-日志与配置信息/${node_number}/hbase 2>/dev/null
        
    }
    done
}

#----------------------------------------------------------------------------------------------------------------------------------------------
# Nginx相关日志和配置文件
nginx_log(){
    for node in ${ALL_NODES}; do
    {
        node_number=$(dbcfgtool.sh -q | grep ${node} | grep -E '^node[0-9]+=')
        # 提前建立好相应文件夹
        mkdir -p ${OUTPUT_PATH_MASTER}/01-日志与配置信息/${node_number}/nginx        
        # 收集指定节点的 Nginx 日志并传输到 node1       
        scp -r root@${node}:/var/log/dblog/nginx/ ${OUTPUT_PATH_MASTER}/01-日志与配置信息/${node_number}/nginx 2>/dev/null
        scp root@${node}:/bigdata/salut/components/Nginx/nginx/conf/nginx.conf* ${OUTPUT_PATH_MASTER}/01-日志与配置信息/${node_number}/nginx 2>/dev/null               
        
    }
    done
}

#----------------------------------------------------------------------------------------------------------------------------------------------
# PostgreSQL相关日志和配置文件
pg_log(){
    for node in ${ALL_NODES}; do
    {
        node_number=$(dbcfgtool.sh -q | grep ${node} | grep -E '^node[0-9]+=')
        # 提前建立好相应文件夹
        mkdir -p ${OUTPUT_PATH_MASTER}/01-日志与配置信息/${node_number}/postgreSQL        
        # 收集指定节点的 PostgreSQL 日志并传输到 node1       
        scp -r root@${node}:/var/log/dblog/pg_log/ ${OUTPUT_PATH_MASTER}/01-日志与配置信息/${node_number}/postgreSQL 2>/dev/null             
        
    }
    done
}

#----------------------------------------------------------------------------------------------------------------------------------------------
# HDFS相关日志和配置文件
hdfs_log(){
    for node in ${ALL_NODES}; do
    {
        node_number=$(dbcfgtool.sh -q | grep ${node} | grep -E '^node[0-9]+=')
        # 提前建立好相应文件夹
        mkdir -p ${OUTPUT_PATH_MASTER}/01-日志与配置信息/${node_number}/hdfs        
        # 收集指定节点的 HDFS 日志并传输到 node1       
        scp root@${node}:/var/log/dblog/hadoop/hadoop-root-datanode-node* ${OUTPUT_PATH_MASTER}/01-日志与配置信息/${node_number}/hdfs 2>/dev/null
        scp root@${node}:/var/log/dblog/hadoop/hadoop-root-namenode-node* ${OUTPUT_PATH_MASTER}/01-日志与配置信息/${node_number}/hdfs 2>/dev/null
        scp root@${node}:/bigdata/salut/components/hadoop/etc/hadoop/hdfs-site* ${OUTPUT_PATH_MASTER}/01-日志与配置信息/${node_number}/hdfs 2>/dev/null
        scp root@${node}:/bigdata/salut/components/hadoop/etc_second/hadoop/hdfs-site* ${OUTPUT_PATH_MASTER}/01-日志与配置信息/${node_number}/hdfs 2>/dev/null
        
    }
    done
}

#----------------------------------------------------------------------------------------------------------------------------------------------
# GP相关日志和配置文件
gp_log(){
    for node in ${ALL_NODES}; do
    {
        node_number=$(dbcfgtool.sh -q | grep ${node} | grep -E '^node[0-9]+=')
        # 提前建立好相应文件夹
        mkdir -p ${OUTPUT_PATH_MASTER}/01-日志与配置信息/${node_number}/greenplum        
        # 收集指定节点的 GP 日志并传输到 node1       
        scp -r root@${node}:/var/log/dblog/greenplum/pg_log/ ${OUTPUT_PATH_MASTER}/01-日志与配置信息/${node_number}/greenplum 2>/dev/null      
        
    }
    done
}

#----------------------------------------------------------------------------------------------------------------------------------------------
# MasterServer、IrisServer、SlaveServer相关日志和配置文件
miserver_log(){
    for node in ${ALL_NODES}; do
    {
        node_number=$(dbcfgtool.sh -q | grep ${node} | grep -E '^node[0-9]+=')
        # 提前建立好相应文件夹
        mkdir -p ${OUTPUT_PATH_MASTER}/01-日志与配置信息/${node_number}/master_iris_slaveserver        
        # 收集指定节点的 MasterServer、IrisServer、SlaveServer 日志并传输到 node1       
        scp root@${node}:/var/log/dblog/salut/thrift/salut-thrift-master.log* ${OUTPUT_PATH_MASTER}/01-日志与配置信息/${node_number}/master_iris_slaveserver 2>/dev/null
        
    }
    done
}

#----------------------------------------------------------------------------------------------------------------------------------------------
# kafka相关日志和配置文件
kafka_log(){
    for node in ${ALL_NODES}; do
    {
        node_number=$(dbcfgtool.sh -q | grep ${node} | grep -E '^node[0-9]+=')
        # 提前建立好相应文件夹
        mkdir -p ${OUTPUT_PATH_MASTER}/01-日志与配置信息/${node_number}/kafka        
        # 收集指定节点的 kafka 日志并传输到 node1       
        scp root@${node}:/var/log/dblog/kafka/controller.log* ${OUTPUT_PATH_MASTER}/01-日志与配置信息/${node_number}/kafka 2>/dev/null
        scp root@${node}:/var/log/dblog/kafka/server.log* ${OUTPUT_PATH_MASTER}/01-日志与配置信息/${node_number}/kafka 2>/dev/null
        scp root@${node}:/bigdata/salut/components/kafka/config/server.properties* ${OUTPUT_PATH_MASTER}/01-日志与配置信息/${node_number}/kafka 2>/dev/null
        
    }
    done
}


####################################################################################################################################################################
####################################################################################################################################################################
# 收集系统日志-参数
####################################################################################################################################################################
####################################################################################################################################################################
# 收集系统日志
collect_system_logs(){
  pid=$$
  another_pid=$(ps -ef | grep -v $pid | grep $0 | grep -v grep | awk '{print $2}')
  if [[ $another_pid ]];then
    echo "检测到此脚本正在运行,如需重新运行请kill掉此前的进程"
    exit 1
  fi

  rm -rf ${OUTPUT_PATH_MASTER} 2>/dev/null

  mkdir -p ${OUTPUT_PATH_MASTER}

  # 信息收集开始
  open
  echo "【注意】仅收集系统日志文件，不影响业务的正常使用！"
 
  system_salog 2>/dev/null
  echo -e "$(date +'%Y-%m-%d %H:%M') 系统\033[0;32m[SA     ]\033[0m日志收集完成"
  wait
  for dir in /home/master-info/sa/*/
  do
    dir_name=$(basename "$dir")
    tar -czvf "/home/master-info/sa/${dir_name}.tar.gz" -C "/home/master-info/sa" "$dir_name" 1>/dev/null
    if [ $? -eq 0 ]; then
        rm -rf "${dir}"
    else
        echo "压缩 ${dir} 失败，未删除原始目录。"
    fi
  done
  wait

  system_cronlog 2>/dev/null
  echo -e "$(date +'%Y-%m-%d %H:%M') 系统\033[0;32m[CRON   ]\033[0m日志收集完成"
  wait
  for dir in /home/master-info/cron/*/
  do
    dir_name=$(basename "$dir")
    tar -czvf "/home/master-info/cron/${dir_name}.tar.gz" -C "/home/master-info/cron" "$dir_name" 1>/dev/null
    if [ $? -eq 0 ]; then
        rm -rf "${dir}"
    else
        echo "压缩 ${dir} 失败，未删除原始目录。"
    fi
  done
  wait

  system_dmesglog 2>/dev/null
  echo -e "$(date +'%Y-%m-%d %H:%M') 系统\033[0;32m[DMESG  ]\033[0m日志收集完成"
  wait
  for dir in /home/master-info/dmesg/*/
  do
    dir_name=$(basename "$dir")
    tar -czvf "/home/master-info/dmesg/${dir_name}.tar.gz" -C "/home/master-info/dmesg" "$dir_name" 1>/dev/null
    if [ $? -eq 0 ]; then
        rm -rf "${dir}"
    else
        echo "压缩 ${dir} 失败，未删除原始目录。"
    fi
  done
  wait

  system_messagelog 2>/dev/null
  echo -e "$(date +'%Y-%m-%d %H:%M') 系统\033[0;32m[MESSAGE]\033[0m日志收集完成"
  wait
  for dir in /home/master-info/message/*/
  do
    dir_name=$(basename "$dir")
    tar -czvf "/home/master-info/message/${dir_name}.tar.gz" -C "/home/master-info/message" "$dir_name" 1>/dev/null
    if [ $? -eq 0 ]; then
        rm -rf "${dir}"
    else
        echo "压缩 ${dir} 失败，未删除原始目录。"
    fi
  done
  wait

  # 总打包
  tar zcvf /home/Collect-SystemLog-`date +%Y-%m-%d-%H-%M-%S`.tgz -C ${OUTPUT_PATH_MASTER} . 1>/dev/null
  wait

  finish
  echo "############################# COLLECTION INFO ##################################"
  echo -e "取包地址：[cd /home]，压缩包名称：\033[0;31mCollect-SystemLog-yyyy-mm-dd-h-m-s.tgz\033[0m"
  echo "################################################################################"
  
  # 删除遗留文件
  rm -rf ${OUTPUT_PATH_MASTER}
}

# 收集sa日志，sa日志收集必须版本对版本（centos6.5收集的必须centos6.5查看）
system_salog(){
  mkdir -p ${PATH_OF_SA}
  for node in ${ALL_NODES};do
  {
    node_number=$(dbcfgtool.sh -q | grep ${node} | grep -E '^node[0-9]+=')
    ssh "${node}" "ls -l --time-style='+%Y-%m-%d %H:%M' /var/log/sa >/var/log/sa/sa_time.txt"
    ssh "${node}" "scp -r /var/log/sa root@node1:${PATH_OF_SA}/${node_number}"
    ssh "${node}" "rm -f /var/log/sa/sa_time.txt"
  }
  done
  wait
}

system_messagelog(){
  for node in ${ALL_NODES};do
  {
    node_number=$(dbcfgtool.sh -q | grep ${node} | grep -E '^node[0-9]+=')
    mkdir -p ${PATH_OF_MESSAGE}/${node_number}
    scp root@${node}:/var/log/message* ${PATH_OF_MESSAGE}/${node_number} >/dev/null 2>&1 &
  }
  done
  wait
}

system_cronlog(){
  for node in ${ALL_NODES};do
  {
    node_number=$(dbcfgtool.sh -q | grep ${node} | grep -E '^node[0-9]+=')
    mkdir -p ${PATH_OF_CRON}/${node_number}
    scp root@${node}:/var/log/cron* ${PATH_OF_CRON}/${node_number} >/dev/null 2>&1 &
  }
  done
  wait
}

system_dmesglog(){
  for node in ${ALL_NODES};do
  {
    node_number=$(dbcfgtool.sh -q | grep ${node} | grep -E '^node[0-9]+=')
    mkdir -p ${PATH_OF_DMESG}/${node_number}
    scp root@${node}:/var/log/dmes* ${PATH_OF_DMESG}/${node_number} >/dev/null 2>&1 &
  }
  done
  wait
}

# CPU信息
cpu_info(){
    echo "-- 【CPU信息，查询命令：cat /proc/cpuinfo】："
    local model=$(cat /proc/cpuinfo | grep name | sort | uniq | awk -F : '{print $2}')
    local total_core=$(cat /proc/cpuinfo | grep processor | wc -l)
    local cpu_num=$(cat /proc/cpuinfo | grep 'physical id' | sort | uniq | wc -l)
    echo "CPU型号：${model}"
    echo "逻辑CPU总数：${total_core}"
    echo "物理CPU数量：${cpu_num}"
    echo
}

# 端口号信息
port_number(){
    echo "--【各端口号占用情况信息，查询命令：netstat -anp | grep 端口号】："
    echo "--【端口号握手状态】：netstat -tunlp | grep 端口号"
    local var
	for var in ${!serverArray[@]}
	do
		# var为索引名称
		local cols=(${serverArray[$var]})
		# cols为内层数组
		for ((col=0; col<${#cols[@]}; col=col+1))
		do
		    local port=${cols[$col]}
			if [[ ${port} != null ]];then
				#握手连接测试
				netstat -tunlp | grep ${port} >/dev/null
				if [ $? != 0 ];then
					echo "${var}: ${port}端口握手失败"
				else
				    echo "-- ${port}端口号占用情况：${var} 服务"
				    netstat -anp | grep ${port}
				fi
				echo
                echo
			fi
		done
	done
}

# 内存信息
memory_info(){
    echo "-- 【内存详情，查询命令：free -g】："
	free -g
    echo
    echo "-- 【进程占用，查询命令：ps aux --sort=-%mem】："
    ps aux --sort=-%mem
    echo
    port_number
}

# 线程进程信息
thread_info(){
    echo "--【进程总数，查询命令：ps -e h | wc -l】："
    ps -e h | wc -l
    echo
    echo "--【进程连接数，查询命令：netstat -antp】："
    netstat -antp | awk '{if($7!="-" && NR>2){s[$7]+=1}}END{for(i in s){printf("%-30s%s\n",i,s[i])}}' | sort -rnk 2
    echo
    echo "--【线程总数，查询命令：ps -e H h | wc -l】："
    ps -e H h | wc -l
    echo
    echo "--【线程数】："
    for i in $(ls -l /proc/ 2>/dev/null | awk '{print $9}' | grep -E '[0-9]+'); do
        tn=$(ls -l /proc/$i/task 2>/dev/null|wc -l)
        if [ $tn -gt 10 ];then
            printf "%-15s%s\n" $i $tn>>/tmp/tn
        fi
    done
    cat /tmp/tn | sort -nrk 2;rm -f /tmp/tn
}

fetch_server_disk_and_mem() {
    echo "~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~"
    echo "标准 DBF 服务器硬盘规格：[1*3.2T SSD + 2*4T]，磁盘：sda >= 3.5T，sdb >= 3.5T，内存：MemTotal >= 250GB；"
    echo "~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~"
    echo "标准 DBE 服务器硬盘规格：[    无 SSD + 1*4T]，磁盘：sda >= 3.5T，             内存：MemTotal >= 120GB；"
    echo "标准 DBE 服务器硬盘规格：[    无 SSD + 2*2T]，磁盘：sda >= 1.5T，sdb >= 1.5T，内存：MemTotal >= 120GB；"
    echo "~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~"
    echo "其余规格均为非标准大数据服务器！！！"
    echo "~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~"
    echo
    cat /proc/partitions | grep -E "\<sd[a-z]?\>" | awk '{print $4":",(($3/1024/1024))"GB"}'
    cat /proc/meminfo |grep -w MemTotal|awk '{print $1, (($2/1024/1024))"GB"}'
    echo
}

# 磁盘信息
disk_info(){
    echo "-- 【磁盘内存规格，按照标配磁盘空间的80%设置阈值】："
    fetch_server_disk_and_mem
    echo
    echo "-- 【分区信息，查询命令：lsblk】："
    lsblk
    echo
    echo "-- 【UUID信息，查询命令：blkid】："
    blkid 2>/dev/null
    echo
    echo "-- 【挂载信息，查询命令：cat /etc/fstab】："
    cat /etc/fstab
    echo
    echo "-- 【磁盘占用，查询命令：df -h】："
	df -h
    echo
    echo "-- 【磁盘类型，查询命令：lsscsi -s】："
    lsscsi -s  2>/dev/null
    echo
    lsblk -d -o NAME,TRAN,model 2>/dev/null
    echo
}

disk_safe(){
    local temp_path=$1
    echo "-- 【磁盘健康状态】："
    sd_devices=$(ls /dev/sd* | grep -oE '^/dev/sd[a-z]$' 2>/dev/null)
    for device in $sd_devices
    do
        echo "【smartctl -H $device】"
        smartctl -H "$device"
    done
    echo
    echo "-- 【磁盘详细信息】："
    sd_devices=$(ls /dev/sd* | grep -oE '^/dev/sd[a-z]$' 2>/dev/null)
    for device in $sd_devices
    do
        echo "【smartctl -a $device】"
        smartctl -a "$device"
    done
    echo
    echo "-- 【磁盘短期测试】：短期测试可能没有收录信息，正常情况"
    sd_devices=$(ls /dev/sd* | grep -oE '^/dev/sd[a-z]$' 2>/dev/null)
    for device in $sd_devices
    do
        echo "【smartctl -t short $device】"
        smartctl -t short "$device"
    done
    echo
    sleep 60
    echo "-- 【磁盘测试结果】："
    sd_devices=$(ls /dev/sd* | grep -oE '^/dev/sd[a-z]$' 2>/dev/null)
    for device in $sd_devices
    do
        echo "【smartctl -l selftest $device】"
        smartctl -l selftest "$device"
    done
    echo
    echo "-- 【磁盘错误日志】："
    sd_devices=$(ls /dev/sd* | grep -oE '^/dev/sd[a-z]$' 2>/dev/null)
    for device in $sd_devices
    do
        echo "【smartctl -l error $device】"
        smartctl -l error "$device"
    done
    echo
    echo "-- 【固态详细信息】："
    nvme_devices=$(ls /dev/nvme0n* | grep -oE '^/dev/nvme0n[1-9]$' 2>/dev/null)
    if [ -n "$nvme_devices" ]; then
        for device in $nvme_devices
        do
            echo "【smartctl -a $device】"
            smartctl -d nvme,0xffffffff -x "$device"
        done
    else
        echo "本台服务器未发现插入固态"
    fi
    echo
    echo "-- 【磁盘读写速度测试】："
    # 测试磁盘写速度
    dd if=/dev/zero of=${temp_path}/.disk_read_write.log bs=4k count=250000 oflag=direct 2>&1
    # 测试磁盘都速度
    dd if=${temp_path}/.disk_read_write.log of=/dev/null bs=4k count=250000 iflag=direct 2>&1
    rm -rf ${temp_path}/.disk_read_write.log
    echo
    echo
    echo "-- 【磁盘读写速度测试】："
    iostat
    echo
    echo "-- 【磁盘IO信息，查询命令：iotop -o -b -n 10 -d 5】："
	iotop -o -b -n 10 -d 5
    echo
}

#查询文件目录大小
check_big(){
    local depth2dirs=$(printf "%s\n%s\n%s\n%s" "/" "/bigdata/salut/components" "/var" "/var/log/dblog")
    local line
    while IFS= read -r line; do
        local cmd
        if [[ "$line" == "/" ]]; then
            cmd="du -h --max-depth=2 --exclude=/proc ${line}  | sort -hr"
        else
            cmd="du -h --max-depth=2 ${line} | sort -hr"
        fi
        echo "=================================================== 【${line} 目录下2级目录】"
        echo "--${line}目录下2级目录所有文件大小，查询命令：${cmd}"
        eval "${cmd}" 2>&1
        echo
        echo
        echo
        echo
    done <<< "$depth2dirs"

    local depth3dirs=$(printf "%s\n%s" "/bigdata" "/data2")
    while IFS= read -r line; do
        local cmd="du -h --max-depth=3 ${line} | sort -hr"
        echo "=================================================== 【${line} 目录下3级目录】"
        echo "--${line}目录下3级目录所有文件大小，查询命令：${cmd}"
        eval "${cmd}" 2>&1
        echo
        echo
        echo
        echo
    done <<< "$depth3dirs"
 }

 # 查询系统版本
system_info(){
    echo "-- 【系统版本，查询命令：cat /etc/redhat-release & cat /etc/os-release & nkvers】："
    # 查询 CentOS 系统信息
    if [ -f /etc/redhat-release ]; then
        cat /etc/redhat-release  2>&1
    # 查询欧拉系统信息
    elif [ -f /etc/os-release ]; then
        cat /etc/os-release  2>&1
    # 查询麒麟系统信息
    elif command -v nkvers >/dev/null 2>&1; then
        nkvers  2>&1
    else
        echo "无法确定操作系统类型"
    fi
    echo
}

# 驱动信息
drive_info(){
    echo "-- 【驱动信息，查询命令：（服务器）ENVmonitor -g -swver & （板卡）ctrl_cli/en_mgmt/display hwversion】："
	ENVmonitor -g -swver 2>&1
    echo
}

# 时间信息
time_info(){
    echo "-- 【系统运行时间，查询命令：uptime】："
    uptime 2>&1
    echo
    echo "-- 【本地时间，查询命令：date】："
	date 2>&1
    echo
    echo "-- 【RTC时间，查询命令：timedatectl status】："
    timedatectl status 2>&1
    echo
}

# 系统服务信息
system_server_info(){
    echo "-- 【系统服务信息，查询命令：（欧拉/centos7以上）systemctl | grep exited & （centos7以下）chkconfig --list | grep '3:on'】："
    # 判断是否为欧拉系统
    if cat /etc/os-release | grep -q "openEuler"; then
        echo "---- 欧拉系统："
        systemctl | grep exited

  # 判断是否为CentOS系统
    elif cat /etc/redhat-release | grep -q "CentOS"; then
        #echo "---- centos系统："
        # 获取CentOS的版本号
        centos_version=$(cat /etc/redhat-release | sed 's/[^0-9\.]*//g' | cut -d. -f1)
        # 判断CentOS版本
        if [ "$centos_version" -lt 7 ]; then
            echo "---- centos系统：版本低于7.0："
            chkconfig --list | grep '3:on'
        else
            echo "---- centos系统：版本高于7.0："
            systemctl | grep exited
        fi
    # 如果不是上述系统，输出不支持的信息
    else
        echo "查询到未知系统，系统服务信息不支持查询"
    fi
    echo
    echo "-- 【系统日志错误信息，查询命令：cat /var/log/messages | grep ERROR】："
	cat /var/log/messages | grep ERROR
    echo
    echo "-- 【系统崩溃错误信息，查询命令：dmesg | grep ERROR】："
	dmesg | grep ERROR
    echo
    echo "-- 【磁盘固态健康信息，查询命令：dmesg | grep 盘符名称】："
	dmesg | grep sd
    echo "--"
    echo "--"
    dmesg | grep EXT4
    echo "--"
    echo "--"
    dmesg | grep nvme
    echo
    echo "-- 【外围设备信息，查询命令：iotop -o -b -n 10 -d 5】："
	lspci | grep ERROR
    echo
}

# ip信息
ip_info(){
    echo "-- 【网络ip信息，查询命令：ip addr】："
    ip addr
    echo
    ifconfig
    echo
    echo "-- 【网络速度，查询命令：sar -n DEV 1 10】："
    sar -n DEV 1 10
    echo
}

# 防火墙信息
firewall_info(){
    echo "=================================================== 【$(hostname)_防火墙信息】"
    echo "-- 【防火墙详情，查询命令：iptables -L -n】："
    iptables -L -n
    echo
}

# 单台节点硬件信息
node_check(){
    local output_dir=$1
    rm -rf ${output_dir} 2>/dev/null
    mkdir -p ${output_dir}
    local temp_path="${output_dir}/temp"
    mkdir -p ${temp_path}

    #硬件信息
    cpu_info >> "${output_dir}/${HOSTNAME}_01进程详情信息.txt" 2>/dev/null
    memory_info >> "${output_dir}/${HOSTNAME}_01进程详情信息.txt" 2>/dev/null
    thread_info >> "${output_dir}/${HOSTNAME}_02线程数量信息.txt" 2>/dev/null
    disk_info >> "${output_dir}/${HOSTNAME}_03磁盘基本信息.txt" 2>/dev/null
    disk_safe "$temp_path" >> ${output_dir}/"${HOSTNAME}_04磁盘健康信息.txt" 2>/dev/null
    check_big >> ${output_dir}/"${HOSTNAME}_05磁盘占用信息.txt" 2>/dev/null
    system_info >> ${output_dir}/"${HOSTNAME}_06系统基本信息.txt" 2>/dev/null
    drive_info >> ${output_dir}/"${HOSTNAME}_06系统基本信息.txt" 2>/dev/null
    time_info >> ${output_dir}/"${HOSTNAME}_06系统基本信息.txt" 2>/dev/null
    system_server_info >> ${output_dir}/"${HOSTNAME}_06系统基本信息.txt" 2>/dev/null
    ip_info >> ${output_dir}/"${HOSTNAME}_06系统基本信息.txt" 2>/dev/null
    firewall_info >> ${output_dir}/"${HOSTNAME}_07防火墙信息.txt" 2>/dev/null

    # sysctl.conf内核参数文件
    cp -r /etc/sysctl.conf* ${output_dir} 2>/dev/null
    # 清理节点目录下的临时文件
    rm -rf ${temp_path}

    if [[ "$CURRENT_IP" != "$MASTER_IP" ]]; then
        scp -r ${output_dir}/* "root@${MASTER_IP}:${output_dir}/" 2>/dev/null
    fi
}

####################################################################################################################################################################
####################################################################################################################################################################
# 收集备份日志信息
####################################################################################################################################################################
####################################################################################################################################################################
# 配置项
CURRENT_DIR="$(cd "$( dirname "${BASH_SOURCE[0]}" )" >/dev/null 2>&1 && pwd)"
COLLECT_LOG_NANE=Collect-BakLog-`date +%Y-%m-%d-%H-%M-%S`
COLLECT_LOG_DIR=$CURRENT_DIR/$COLLECT_LOG_NANE
BIGDATA=/bigdata/salut
VAR_CONF="$BIGDATA"/conf
CFGCHG_BIGDATA="$VAR_CONF"/db_bigdata.conf
BACKUP_LOG_DIR=/var/log/dblog/backupLog
SALUT_LOG_DIR=/var/log/dblog/salut
SALUT_WEB_DIR=/var/log/dblog/salut/web
SALUT_THRIFT_DIR=/var/log/dblog/salut/thrift
###################################################################################################################################
# 备份日志主函数
log_by_time(){
  num_args=$#
  if [ $num_args -eq 0 ]; then
    echo
    echo -e "\033[0;33m 格式:   sh $SCRIPT_NAME -tlog [days] [date]\033[0m"
    echo
    echo -e "\033[0;35m ******************************************************************************************************\033[0m"
    echo -e "\033[0;35m|       参数  必填   非必填        详情解释                                                            |\033[0m"
    echo -e "\033[0;35m|      -tlog [days] [date]      [days]：必填项，收集最近 [days] 天的备份日志                           | \033[0m"
    echo -e "\033[0;35m|                               [date]：非必填项，格式：YYYY-MM-DD                                     | \033[0m"
    echo -e "\033[0;35m|                                                                                                      | \033[0m"
    echo -e "\033[0;35m|      -tlog   3                从当前时间开始，收集最近 3 天的备份日志                                | \033[0m"
    echo -e "\033[0;35m|      -tlog   5   2024-06-01   从 2024-06-01 开始，收集之后 5 天的备份日志(2024-06-01 ~~ 2024-06-05)  | \033[0m"
    echo -e "\033[0;35m ******************************************************************************************************\033[0m"
    exit 1
  elif [ $num_args -ge 2 ]; then
    startDay=$2
  fi
  days=$1
  if ! [[ $days =~ ^[0-9]+$ ]]; then
    echo "$(date +'%Y-%m-%d %H:%M') \033[0;31m输入的天数非法\033[0m"
    exit 1
  fi
    open
    echo "【注意】仅收集备份日志文件，不影响业务的正常使用！"
  if [ -z $startDay ]; then
    echo "$(date +'%Y-%m-%d %H:%M') 收集所有节点最近$days天的备份日志"
  else
    echo "$(date +'%Y-%m-%d %H:%M') 收集所有节点$startDay开始之后$days天的备份日志"
  fi


  # 创建收集日志的目录
  if [ -d $COLLECT_LOG_DIR ]; then
    rm -rf $COLLECT_LOG_DIR
  fi
  mkdir -p $COLLECT_LOG_DIR

  # 读取配置文件，获取节点信息
  for line in `cat ${CFGCHG_BIGDATA} | grep ^node`; do
    #node=`echo $line | awk -F = '{print $1}'`
    node=`echo $line `
    ip=`echo $line | awk -F = '{print $2}'`
    echo "$(date +'%Y-%m-%d %H:%M') 开始收集$node的备份日志"
    collectNodeDir=$COLLECT_LOG_DIR/$node
    mkdir -p $collectNodeDir


    # 根据参数数量处理不同的日志收集逻辑
    if [[ $days -ne 0 ]]; then
      if [ -z $startDay ]; then
        collectFileList=$(ssh $ip "ls -r $BACKUP_LOG_DIR/dblog-*.tar.gz | head -n $days")
      else
        fileList=$(ssh $ip "ls -v $BACKUP_LOG_DIR/dblog-*.tar.gz")
        if [[ $fileList == *"$startDay"* ]]; then
          collectFileList=$(tail -n +$(echo "$fileList" | grep -n "$startDay" | cut -d ":" -f 1) <<< "$fileList" | head -n $days)
        else
          echo "$(date +'%Y-%m-%d %H:%M') 没有找到$node在$startDay的备份日志"
        fi
      fi
      if [ -n "$collectFileList" ]; then
        for file in $collectFileList; do
          scp -r $ip:$file $collectNodeDir > /dev/null
        done
      fi
    fi

    
    scp -r $ip:$SALUT_LOG_DIR/web $collectNodeDir > /dev/null
    cd $collectNodeDir;tar -zcvf web.tar.gz web > /dev/null
    rm -rf $collectNodeDir/web

    scp -r $ip:$SALUT_LOG_DIR/thrift $collectNodeDir > /dev/null
    cd $collectNodeDir;tar -zcvf thrift.tar.gz thrift > /dev/null
    rm -rf $collectNodeDir/thrift

    echo "$(date +'%Y-%m-%d %H:%M') 完成收集$node的备份日志"
  done
  cd $CURRENT_DIR;tar -zcvf /home/$COLLECT_LOG_NANE.tar.gz $COLLECT_LOG_NANE > /dev/null
  rm -rf $COLLECT_LOG_DIR

  finish
  echo "############################# COLLECTION INFO ##################################"
  echo -e "取包地址：[cd /home]，压缩包名称：\033[0;31mCollect-BakLog-yyyy-mm-dd-h-m-s.tgz\033[0m"
  echo "################################################################################"
}








####################################################################################################################################################################
####################################################################################################################################################################
# 判断磁盘硬件信息
####################################################################################################################################################################
####################################################################################################################################################################
# 磁盘规格
get_disk_size() {
    local disk=$1
    local size=$(cat /proc/partitions | grep -E "\<sd[a-z]?\>" | awk -v disk="$disk" '$4 == disk {print (($3/1024/1024/1024))}')
    echo "$size"
}
# SSD 规格
get_ssd_size() {
    local size=$(cat /proc/partitions | awk '/nvme/ {print ($3/1024/1024/1024)}')
    echo "$size"
}
# 内存规格
get_mem_size() {
    local size=$(cat /proc/meminfo | grep -w MemTotal | awk '{print (($2/1024/1024))}')
    echo "$size"
}

# 浮点数比较函数
float_compare() {
    awk "BEGIN {exit !($1 $2 $3)}" 2>/dev/null
}
# 判断服务器类型
check_server_type() {
    local sda_size=$(get_disk_size "sda")
    local sdb_size=$(get_disk_size "sdb")
    local ssd_size=$(get_ssd_size)
    local mem_size=$(get_mem_size)
    # DBF
    if  float_compare "$sda_size" ">=" "3.6" && \
        float_compare "$sdb_size" ">=" "3.6" && \
        float_compare "$ssd_size" ">=" "2.9" && \
        float_compare "$mem_size" ">=" "250"; then
        server_type="DBF"
        echo -e "\033[0;36mStart to check disk specification ...\033[0m"
        echo
        echo -e "当前硬件符合：\033[0;35m大数据【$server_type】规格\033[0m "
        echo
        DBF_info
    # DBE_A elif
    elif  float_compare "$sda_size" ">=" "1.8" && float_compare "$sda_size" "<" "2" && \
          float_compare "$sdb_size" ">=" "1.8" && float_compare "$sdb_size" "<" "2" && \
          float_compare "$mem_size" ">=" "120"; then
          server_type="DBE_A"
          echo -e "\033[0;36mStart to check disk specification ...\033[0m"
          echo
          echo -e "当前硬件符合：\033[0;35m大数据【${server_type:0:3}】 规格\033[0m"
          echo
          DBE_A_info
    # DBE_B
    elif  float_compare "$sda_size" ">=" "3.6" && \
          float_compare "$mem_size" ">=" "120"; then
          server_type="DBE_B"
          echo -e "\033[0;36mStart to check disk specification ...\033[0m"
          echo
          echo -e "当前硬件符合：\033[0;35m大数据【${server_type:0:3}】 规格\033[0m"
          echo
          DBE_B_info
    # 第三方
    else
        server_type="非标准"
        echo -e "\033[0;36mStart to check disk specification ...\033[0m"
        echo
        echo -e "当前硬件属于：\033[0;35m【$server_type】规格\033[0m"
        echo
        other_there
    fi
}

# 调用函数
DBE_A_info(){
    echo "----------------------------------------------------------------------" 
    echo -e "\033[0;31m【当前服务器硬件的实际规格】：\033[0m"
    cat /proc/partitions | grep -E "\<sd[a-z]?\>" | awk '{print $4" "(($3/1024/1024/1024))}' | while read -r disk size; do
    echo "磁盘：$disk = $size TB"
    done
    echo "内存: Mem = $mem_size GB"
    if [ -z "$ssd_size" ] || [ "$(echo "$ssd_size == 0" | bc)" -eq 1 ]; then
    echo "固态：SSD = 不存在"
    else
    echo "固态：SSD = $ssd_size TB"
    fi
    echo
    echo -e "\033[0;31m【当前服务器分区的实际详情】：\033[0m"
    lsblk
    echo
    echo "----------------------------------------------------------------------"
    echo -e "\033[0;33m【DBE 服务器硬件的标准规格】：[ 2*2T 磁盘 + 4*32G 内存条 +  无  SSD ]\033[0m"
    echo "磁盘：sda = 2.0 TB  (阈值：sda >= 1.8 TB)"
    echo "磁盘：sdb = 2.0 TB  (阈值：sdb >= 1.8 TB)"
    echo "内存：Mem = 128 GB  (阈值：Mem >= 120 TB)"
    echo
    echo -e "\033[0;33m【DBE 服务器分区的标准详情】：\033[0m"
    echo "NAME                           MAJ:MIN RM  SIZE RO TYPE MOUNTPOINTS"
    echo "sda                              8:0    0  1.8T  0 disk"
    echo "└─sda1                           8:1    0  1.8T  0 part"
    echo "  └─vg_bigdata-LogVol01 (dm-0) 253:0    0  3.4T  0 lvm  /"
    echo "sdb                              8:16   0  1.8T  0 disk"
    echo "├─sdb1                           8:17   0  500M  0 part /boot"
    echo "├─sdb2                           8:18   0   16G  0 part [SWAP]"
    echo "└─sdb3                           8:19   0  1.8T  0 part"
    echo "  ├─vg_bigdata-LogVol01 (dm-0) 253:0    0  3.4T  0 lvm  /"
    echo "  └─vg_bigdata-LogVol00 (dm-1) 253:1    0  200G  0 lvm  /var"
    echo
    echo "----------------------------------------------------------------------"
    echo
    echo -e "\033[0;36m... Finished check disk specification\033[0m"
    echo
}

DBE_B_info(){
    echo "----------------------------------------------------------------------" 
    echo -e "\033[0;31m【当前服务器硬件的实际规格】：\033[0m"
    cat /proc/partitions | grep -E "\<sd[a-z]?\>" | awk '{print $4" "(($3/1024/1024/1024))}' | while read -r disk size; do
    echo "磁盘：$disk = $size TB"
    done
    echo "内存: Mem = $mem_size GB"
    if [ -z "$ssd_size" ] || [ "$(echo "$ssd_size == 0" | bc)" -eq 1 ]; then
    echo "固态：SSD = 不存在"
    else
    echo "固态：SSD = $ssd_size TB"
    fi
    echo
    echo -e "\033[0;31m【当前服务器分区的实际详情】：\033[0m"
    lsblk
    echo
    echo "----------------------------------------------------------------------"
    echo -e "\033[0;33m【DBE 服务器硬件的标准规格】：[ 1*4T 磁盘 + 4*32G 内存条 +  无  SSD ]\033[0m"
    echo "磁盘：sda = 4.0 TB  (阈值：sda >= 3.6 TB)"
    echo "内存：Mem = 128 GB  (阈值：Mem >= 120 TB)"
    echo
    echo -e "\033[0;33m【DBE 服务器分区的标准详情】：\033[0m"
    echo "NAME    MAJ:MIN RM  SIZE RO TYPE MOUNTPOINTS"
    echo "sda       8:0    0  3.6T  0 disk"
    echo "├─sda1    8:1    0  200M  0 part /boot/efi"
    echo "├─sda2    8:2    0  500M  0 part /boot"
    echo "├─sda3    8:3    0  3.4T  0 part /"
    echo "├─sda4    8:4    0  200G  0 part /var"
    echo "└─sda5    8:5    0   16G  0 part [SWAP]"
    echo
    echo "----------------------------------------------------------------------"
    echo
    echo -e "\033[0;36m... Finished collect information\033[0m"
    echo
}

DBF_info(){
    echo "----------------------------------------------------------------------" 
    echo -e "\033[0;31m【当前服务器硬件的实际规格】：\033[0m"
    cat /proc/partitions | grep -E "\<sd[a-z]?\>" | awk '{print $4" "(($3/1024/1024/1024))}' | while read -r disk size; do
    echo "磁盘：$disk = $size TB"
    done
    echo "内存: Mem = $mem_size GB"
    if [ -z "$ssd_size" ] || [ "$(echo "$ssd_size == 0" | bc)" -eq 1 ]; then
    echo "固态：SSD = 不存在"
    else
    echo "固态：SSD = $ssd_size TB"
    fi
    #echo "固态：SSD = $ssd_size TB"
    echo
    echo -e "\033[0;31m【当前服务器分区的实际详情】：\033[0m"
    lsblk
    echo
    echo "----------------------------------------------------------------------"
    echo -e "\033[0;33m【DBF 服务器硬件的标准规格】：[ 2*4T 磁盘 + 8*32G 内存条 + 1*3.2T SSD]\033[0m"
    echo "磁盘：sda = 4.0 TB  (阈值：sda >= 3.6 TB)"
    echo "磁盘：sdb = 4.0 TB  (阈值：sdb >= 3.6 TB)"
    echo "固态：SSD = 3.2 TB  (阈值：SSD >= 2.9 TB)"
    echo "内存：Mem = 256 GB  (阈值：Mem >= 250 TB)"
    echo
    echo -e "\033[0;33m【DBF 服务器分区的标准详情】：\033[0m"
    echo "NAME    MAJ:MIN RM  SIZE RO TYPE MOUNTPOINTS"
    echo "sda       8:0    0  3.6T  0 disk"
    echo "├─sda1    8:1    0  200M  0 part /boot/efi"
    echo "├─sda2    8:2    0  500M  0 part /boot"
    echo "├─sda3    8:3    0  3.4T  0 part /"
    echo "├─sda4    8:4    0  200G  0 part /var"
    echo "└─sda5    8:5    0   32G  0 part [SWAP]"
    echo "sdb       8:16   0  3.6T  0 disk /data2"
    echo "nvme0n1 259:0    0  2.9T  0 disk /index"
    echo
    echo "----------------------------------------------------------------------"
    echo
    echo -e "\033[0;36m... Finished check disk specification\033[0m"
    echo
}

other_there(){
    echo "----------------------------------------------------------------------"
    echo -e "\033[0;31m【非标服务器硬件的实际规格】：\033[0m"
    cat /proc/partitions | grep -E "\<sd[a-z]?\>" | awk '{print $4" "(($3/1024/1024/1024))}' | while read -r disk size; do
    echo "磁盘：$disk = $size TB"
    done
    echo "内存: Mem = $mem_size GB"
    if [ -z "$ssd_size" ] || [ "$(echo "$ssd_size == 0" | bc)" -eq 1 ]; then
    echo "固态：SSD = 不存在"
    else
    echo "固态：SSD = $ssd_size TB"
    fi
    #echo "固态：SSD = $ssd_size TB"
    echo
    echo -e "\033[0;31m【非标服务器分区的实际详情】：\033[0m"
    lsblk
    echo
    echo "****参考我司硬件的标准规格，请确认缺少哪些硬件资源！！****"
    echo
    disk_and_mem_info
    echo
    echo -e "\033[0;36m... Finished check disk specification\033[0m"
    echo
}

# 标准DBE/DBF 磁盘分区标准
disk_and_mem_info() {
  echo "----------------------------------------------------------------------"
  echo -e "\033[0;35m《我司大数据【DBE/DBF】硬件标准参考》：\033[0m"
  echo
  echo -e "\033[34;1m【DBF 标准服务器】：\033[0m"
  echo -e "\033[32;1m--规格： [ 2*4T 磁盘 + 8*32G 内存条 + 1*3.2T SSD]\033[0m"
  echo "  磁盘：sda = 4.0T   (实际可用值：sda >= 3.6 TB)"
  echo "  磁盘：sdb = 4.0T   (实际可用值：sdb >= 3.6 TB)"
  echo "  内存：Mem = 256GB  (实际可用值：Mem >= 250 GB)"
  echo "  固态：SSD = 3.2T   (实际可用值：SSD >= 2.9 TB)"
  echo 
  echo -e "\033[34;1m【DBE 标准服务器】：\033[0m"
  echo -e "\033[32;1m--规格1：[ 1*4T 磁盘 + 4*32G 内存条 +  无  SSD ]\033[0m"
  echo "  磁盘：sda = 4.0T   (实际可用值：sda >= 3.6 TB)"
  echo "  内存：Mem = 128GB  (实际可用值：Mem >= 120 GB)"
  echo
  echo -e "\033[32;1m--规格2：[ 2*2T 磁盘 + 4*32G 内存条 +  无  SSD ]\033[0m"
  echo "  磁盘：sda = 2.0T   (实际可用值：sda >= 1.8 TB)"
  echo "  磁盘：sdb = 2.0T   (实际可用值：sdb >= 1.8 TB)"
  echo "  内存：Mem = 128GB  (实际可用值：Mem >= 120 GB)"
  echo 
  echo -e "\033[31;1m【 非 标准服务器】：\033[0m"
  echo -e "\033[31;1m  非我司硬件或不符合我司硬件标准，请联系二线进行评估！\033[0m"
  echo "----------------------------------------------------------------------"
 }












###################################################################################################################################
###################################################################################################################################
###################################################################################################################################
###################################################################################################################################
###################################################################################################################################
###################################################################################################################################
###################################################################################################################################
###################################################################################################################################
###################################################################################################################################
###################################################################################################################################
###################################################################################################################################
###################################################################################################################################
###################################################################################################################################
###################################################################################################################################
###################################################################################################################################
###################################################################################################################################
###################################################################################################################################
###################################################################################################################################
########################################################【界面信息】################################################################
# 界面帮助
help(){
  echo -e "\n【注意】使用 指令+参数 的操作格式如下："
  echo
  echo -e "\033[0;33m eg: $SCRIPT_NAME -参数\033[0m"
  echo -e "\033[0;35m *********************************************\033[0m"
  echo -e "\033[0;35m|       参数                对应功能          |\033[0m"
  #echo -e "\033[0;35m|      -anode          收集当前节点详情信息   |\033[0m"
  echo -e "\033[0;35m|      -clog           收集全部节点组件日志   |\033[0m"
  echo -e "\033[0;35m|      -slog           收集全部节点系统日志   |\033[0m"
  echo -e "\033[0;35m|      -tlog           收集全部节点备份日志   |\033[0m"
  echo -e "\033[0;35m|      -standard       大数据硬件的标准规格   |\033[0m"
  echo -e "\033[0;35m|      -standardCheck  检测当前节点硬件规格   |\033[0m"
  echo -e "\033[0;35m *********************************************\033[0m"
}

help_rep(){
  echo -e "\033[0;35m *********************************************\033[0m"
  echo -e "\033[0;35m|       参数                对应功能          |\033[0m"
  #echo -e "\033[0;35m|       anode          收集当前节点详情信息   |\033[0m"
  echo -e "\033[0;35m|       clog           收集全部节点组件日志   |\033[0m"
  echo -e "\033[0;35m|       slog           收集全部节点系统日志   |\033[0m"
  echo -e "\033[0;35m|       tlog           收集全部节点备份日志   |\033[0m"
  echo -e "\033[0;35m|       standard       大数据硬件的标准规格   |\033[0m"
  echo -e "\033[0;35m|       standardCheck  检测当前节点硬件规格   |\033[0m"
  echo -e "\033[0;35m *********************************************\033[0m"
}


# 函数用于提示用户重新输入参数，并执行相应函数
retry_prompt() {
  read -p "请重新输入对应参数: " new_option
  echo
  echo "----------------------------------------------------------------------"
  case "$new_option" in
    clog)
      component_log
      ;;
    slog)
      collect_system_logs
      ;;
    tlog)
      otherArgs=("${@:2}")
      log_by_time "${otherArgs[@]}"
      ;;
    standardCheck)
      check_server_type
	    ;;
    standard)
      disk_and_mem_info
	    ;;
    *)
      echo -e "\033[0;31m[ $new_option ] 为无效参数，请使用上述对话框内的参数格式\033[0m"
      echo
      echo "----------------------------------------------------------------------"
      retry_prompt
      ;;
  esac
}

# 检查是否提供了参数
if [ -z "$1" ]; then
    pid=$$
    another_pid=$(ps -ef | grep -v $pid | grep $0 | grep -v grep | awk '{print $2}')
    if [[ $another_pid ]];then
        echo "检测到此脚本正在运行,如需重新运行请kill掉历史进程==>$another_pid"
        exit 1
    fi
    # 如果没有提供参数，则询问用户是否输入参数
    read -p "巡检基本信息输入 y ，收集其他信息输入 n（默认为 y ）：" choice
    echo
    if [ -z "$choice" ]; then
        choice="y"
    fi
    case "$choice" in
        y|Y)
            main
            ;;
        n|N)
            help_rep
            echo
            echo "----------------------------------------------------------------------"
            read -p "请输入确认好的参数: " option
            echo
            echo "----------------------------------------------------------------------"
            case "$option" in
                info)
                    collect_info
                    ;;
                clog)
                    component_log
                    ;;
                slog)
                    collect_system_logs
                    ;;
                tlog)
                    otherArgs=("${@:2}")
                    log_by_time "${otherArgs[@]}"
                    ;;
                standardCheck)
                    check_server_type
	                ;;
                standard)
                    disk_and_mem_info
	                ;;
                *)
                    echo -e "\033[0;31m[ $option ] 为无效参数，请按照提示选择正确的参数\033[0m"
                    echo
                    echo "----------------------------------------------------------------------"
                    retry_prompt
                    ;;
            esac
            ;;
        *)
            echo -e "\033[0;31m您输入的是无效参数，工具已自动退出！\033[0m"
            echo
            echo "----------------------------------------------------------------------"
            ;;
    esac
else
    # 可根据传入函数名参数单独查询某一块信息
    case $1 in
        "-check")
            main
	        ;;
        "-node")
            node_check "$2"
	        ;;
        "-clog")
            component_log
	        ;;
        "-slog")
            collect_system_logs
	        ;;
        "-tlog")
            otherArgs=("${@:2}")
            log_by_time "${otherArgs[@]}"
            ;;
        "-call")
            collect_component_all
	        ;;
        "-standardCheck")
            check_server_type
	        ;;
        "-standard")
            disk_and_mem_info
	        ;;
	    "-get_statistics_date")
	        get_statistics_date $2
	        ;;
	    "-salut_ssh")
	        salut_ssh "$2" "$3"
	        ;;
        "")
            main # 如果没有提供参数，则默认执行 main 函数
            ;;
        *)
            help
            ;;
    esac
fi